/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package assistant;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import storybook.model.book.Book;
import storybook.tools.LOG;

/**
 * class for Assistant data
 *
 * @author favdb
 */
public class Assistant {

	private static final String TT = "Assistant";

	public static AssistantXml xml;

	public Assistant() {
		//empty
	}

	public Assistant(String filename) {
		init(filename);
	}

	/**
	 * initialize Assistant with fileName XML
	 *
	 * @param fileName
	 */
	public static void init(String fileName) {
		//LOG.trace(TT + ".init(fileName=" + fileName + ")");
		init(new File(fileName));
	}

	/**
	 * initialize Assistant with file XML
	 *
	 * @param xmlFile
	 */
	public static void init(File xmlFile) {
		//LOG.trace(TT + ".init(xmlFile=\"" + (xmlFile == null ? "null" : xmlFile.getAbsolutePath()) + "\")");
		xml = new AssistantXml(xmlFile);
	}

	/**
	 * get the Assistant values as a HTML String
	 *
	 * @param values
	 * @return
	 */
	public static String toHtml(String values) {
		return AssistantXml.getHtml(values);
	}

	/**
	 * convert the Assistant values in HTML to a text String (removes the HTML tags and carriage
	 * return)
	 *
	 * @param values
	 * @return
	 */
	public static String toText(String values) {
		if (values == null || values.isEmpty()) {
			return "";
		}
		String v = toHtml(values).replace("\n", "");
		v = v.replace("<h1>", "").replace("</h1>", "=>");
		v = v.replace("<h2>", "").replace("</h2>", ": ");
		v = v.replace("<h3>", "").replace("</h3>", "=");
		v = v.replace("<b>", "").replace("</b>: ", "=\"");
		v = v.replace("<p>", "").replace("</p>", "\" ");
		return v;
	}

	/**
	 * check if Book has the assistant Text checked
	 *
	 * @param book
	 * @param toString
	 * @return
	 */
	public boolean isExists(Book book, String toString) {
		//to change if book assistant has to check
		return true;
	}

	/**
	 * check if SECTION exists in Assistant
	 *
	 * @param section
	 * @return
	 */
	public boolean isExists(String section) {
		return false;
	}

	/**
	 * get list of options for a combobox or a listbox
	 *
	 * @param item: the item to search
	 *
	 * @return
	 */
	public static List<String> getListOf(String item) {
		//todo get list of combobox options
		List<String> list = new ArrayList<>();
		return list;
	}

	public static String getLastOpened() {
		return getXml().getLastOpened();
	}

	/**
	 * get the Assistant XML
	 *
	 * @return
	 */
	public static AssistantXml getXml() {
		if (xml == null) {
			xml = new AssistantXml(null);
		}
		return xml;
	}

	/**
	 * set the Assistant XML for a local file name
	 *
	 * @param name
	 */
	public static void setXml(String name) {
		File f = new File(name);
		if (AssistantXml.isValid(f)) {
			xml = new AssistantXml(f);
		}
	}

	/**
	 * SECTION types
	 *
	 */
	private static final String BOOK_SECTION = "book, part, chapter, scene, strand";

	public enum SECTION {
		NONE, BOOK, PERSON, LOCATION, ITEM, PLOT;

		@Override
		public String toString() {
			return name().toLowerCase();
		}
	}

	/**
	 * get the SECTION
	 *
	 * @param value
	 * @return
	 */
	public static SECTION getSECTION(String value) {
		//LOG.trace(TT + ".getSECTION(value=" + value + ")");
		if (BOOK_SECTION.contains(value.toLowerCase())) {
			return SECTION.BOOK;
		}
		for (SECTION section : SECTION.values()) {
			if (value.equalsIgnoreCase(section.toString())) {
				return section;
			}
		}
		LOG.err(TT + ".getSECTION unknown \"" + value + "\"");
		return SECTION.BOOK;
	}

	/**
	 * field types
	 */
	public enum TYPE {
		TEXTFIELD, TEXTAREA, LISTBOX, COMBOBOX;

		@Override
		public String toString() {
			return name().toLowerCase();
		}
	}

	public static TYPE getTYPE(String value) {
		for (TYPE type : TYPE.values()) {
			if (value.equalsIgnoreCase(type.name())) {
				return type;
			}
		}
		return TYPE.TEXTFIELD;
	}

}
