/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ideabox;

import java.io.Serializable;
import java.util.Objects;

/**
 * class de définition d'une idée de la boîte à idées
 *
 * @author favdb
 */
public class IdeaX implements Serializable, Comparable<IdeaX> {

	private Long id = -1L;
	private String creation = "", maj = "";
	private String name;
	private Integer status;
	private String description = "", notes = "";

	public IdeaX() {
		// standard declaration
	}

	public Long getId() {
		return this.id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getCreation() {
		return this.creation;
	}

	public void setCreation(String creation) {
		this.creation = creation;
	}

	public String getMaj() {
		return this.maj;
	}

	public void setMaj(String maj) {
		this.maj = maj;
	}

	public Integer getStatus() {
		return this.status;
	}

	public void setId(Integer status) {
		this.status = status;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getNotes() {
		return this.notes;
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	public String toString() {
		String sep = ", ";
		return name + "{"
				+ id.toString() + sep
				+ status.toString() + sep
				+ "\"" + description + "\"" + sep
				+ "\"" + notes + "\"" + "}";
	}

	public String toHtml() {
		String sep = "<br>";
		return name + ": " + sep
				+ "id=" + id.toString() + sep
				+ "creation=" + creation + sep
				+ "maj=" + maj + sep
				+ "status=" + status.toString() + sep
				+ "description=\"" + description + "\"" + sep
				+ "notes=\"" + notes + "\"";
	}

	@Override
	public int compareTo(IdeaX o) {
		return getName().compareTo(o.getName());
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = hash * 31 + (id != null ? id.hashCode() : 0);
		hash = hash * 31 + (name != null ? name.hashCode() : 0);
		hash = hash * 31 + (status != null ? status.hashCode() : 0);
		hash = hash * 31 + (description != null ? description.hashCode() : 0);
		hash = hash * 31 + (notes != null ? notes.hashCode() : 0);
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final IdeaX other = (IdeaX) obj;
		if (!Objects.equals(this.id, other.id)) {
			return false;
		}
		if (!Objects.equals(this.name, other.name)) {
			return false;
		}
		if (!Objects.equals(this.description, other.description)) {
			return false;
		}
		if (!Objects.equals(this.notes, other.notes)) {
			return false;
		}
		return Objects.equals(this.status, other.status);
	}

}
