/*
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the NovaWorx project. Other parts are
 * from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.editors.wys;

import i18n.I18N;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.JEditorPane;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import resources.icons.ICONS;
import resources.icons.IconUtil;

/**
 *
 */
public class HTMLLineBreakAction extends HTMLTextEditAction {

	private static final String TT = "HTMLLineBreakAction";

	public HTMLLineBreakAction(WysiwygEditor editor) {
		super(editor, I18N.getMsg("shef.line_break"));
		setSmallIcon(IconUtil.getIconSmall(ICONS.K.HTML_BR));
		setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.SHIFT_DOWN_MASK, true));
		setShortDescription(I18N.getMsg("shef.line_break"));
	}

	int nbfun = 0;

	@Override
	protected void wysiwygEditPerformed(ActionEvent evt, JEditorPane editor) {
		//LOG.trace(TT + "wysiwygEditPerformed(evt, editor) nbfun=" + nbfun);
		if (nbfun > 0) {
			nbfun = 0;
			return;
		}
		if (evt.getSource() instanceof JMenuItem) {
			nbfun++;
		}
		HTMLDocument document = (HTMLDocument) editor.getDocument();
		int pos = editor.getCaretPosition();
		String elName = document.getParagraphElement(pos).getName();
		HTML.Tag tag = HTML.getTag(elName);
		if (elName.toUpperCase().equals("P-IMPLIED")) {
			tag = HTML.Tag.IMPLIED;
		}
		HTMLEditorKit.InsertHTMLTextAction hta
				= new HTMLEditorKit.InsertHTMLTextAction("insertBR", "<br>", tag, HTML.Tag.BR);
		hta.actionPerformed(evt);
	}

}
