/*
 * Copyright (C) 2020 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a fileCopy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.action;

import i18n.I18N;
import java.io.File;
import javax.swing.JOptionPane;
import static javax.swing.JOptionPane.QUESTION_MESSAGE;
import storybook.App;
import storybook.exim.exporter.ExportBookDlg;
import storybook.exim.exporter.ExportDlg;
import storybook.exim.exporter.ExportOptionsDlg;
import storybook.exim.exporter.ExportStoryboard;
import storybook.exim.exporter.ExportTable;
import storybook.exim.importer.ImportDlg;
import storybook.exim.importer.ImportDocument;
import storybook.exim.importer.ImportEpub;
import storybook.model.H2File;
import storybook.tools.LOG;
import storybook.tools.file.IOUtil;
import storybook.tools.swing.SwingUtil;
import storybook.ui.MainFrame;
import storybook.ui.dialog.ChaptersOrderDlg;
import storybook.ui.dialog.ChooseFileDlg;
import storybook.ui.dialog.preferences.PropertiesDlg;

/**
 *
 * @author favdb
 */
public class MainAction {

	private static final String TT = "MainAction";

	private final MainFrame mainFrame;

	public MainAction(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	/**
	 * Actions about File *
	 */
	public void fileNew() {
		App.getInstance().createNewFile();
	}

	public void fileOpen() {
		mainFrame.cursorSetWaiting();
		App.getInstance().openFile();
		mainFrame.cursorSetDefault();
	}

	public void fileSave() {
		mainFrame.saveFile(true);
	}

	public void fileSaveAs() {
		ChooseFileDlg dlg = new ChooseFileDlg(mainFrame, false);
		dlg.setForceDbExtension(false);
		dlg.setDefaultDBExt(mainFrame.getH2File().getExt());
		dlg.setDefaultPath(mainFrame.getH2File().getPath());
		dlg.setVisible(true);
		if (dlg.isCanceled()) {
			return;
		}
		File outFile = dlg.getFile();
		File inFile = mainFrame.getH2File().getFile();
		String oldPath = mainFrame.getH2File().getPath();
		mainFrame.close(false);
		if (!IOUtil.fileCopy(inFile, outFile)) {
			LOG.err("FileUtils.copyFile(?,?) IOex : ");
		}
		H2File newDB = new H2File(outFile);
		String newPath = newDB.getPath();
		App.getInstance().openFile(newDB, oldPath, newPath);
	}

	public void fileRename() {
		//App.trace("MainAction.fileRename()");
		String m = JOptionPane.showInputDialog(mainFrame,
				I18N.getMsg("project.rename.new.name"),
				I18N.getMsg("project.rename"),
				QUESTION_MESSAGE);
		if (m != null && !m.isEmpty()) {
			m = mainFrame.getH2File().getPath() + File.separator + m + ".osbk";
			File outFile = new File(m);
			if (outFile.exists()) {
				int ret = JOptionPane.showConfirmDialog(mainFrame,
						I18N.getMsg("file.exists", m),
						I18N.getMsg("file.save.overwrite.title"),
						JOptionPane.YES_NO_OPTION);
				if (ret == JOptionPane.NO_OPTION) {
					return;
				}
			}
			App.getInstance().renameFile(mainFrame, outFile);
		}
	}

	public void fileProperties() {
		PropertiesDlg dlg = new PropertiesDlg(mainFrame, false);
		dlg.setVisible(true);
	}

	public void fileImportDoc() {
		//App.trace(TT + ".fileImportDoc()");
		ImportDocument doc = new ImportDocument(mainFrame);
		File file = doc.selectFile();
		if (file != null) {
			doc.setFile(file);
			if (IOUtil.getExtension(file).equals("epub")) {
				ImportEpub.doImport(file);
				return;
			}
			if (doc.openDocument() && doc.openDB()) {
				mainFrame.cursorSetWaiting();
				App.getInstance().importBook(doc);
				mainFrame.cursorSetDefault();
				doc.close();
			}
		}
	}

	/**
	 * Actions about Exports *
	 */
	public void exportNew() {
		ExportDlg.show(mainFrame);
	}

	public void exportBook() {
		ExportBookDlg.show(mainFrame);
	}

	public void exportXml() {
		//select directory
		ExportTable.exportDB(mainFrame);
	}

	public void exportStoryboard() {
		ExportStoryboard.execute(mainFrame);
	}

	public void exportOther() {
		ExportDlg.show(mainFrame);
	}

	public void exportOptions() {
		ExportOptionsDlg.show(mainFrame);
	}

	public void importNew() {
		ImportDlg.show(mainFrame);
	}

	/**
	 * Actions about Chapter *
	 */
	public void chaptersOrderAction() {
		SwingUtil.showModalDialog(new ChaptersOrderDlg(mainFrame), mainFrame);
	}

}
