/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import storybook.ui.Ui;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JTextArea;
import storybook.model.EntityUtil;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Relationship;
import storybook.tools.ListUtil;
import storybook.ui.dialog.RelationsDlg;

/**
 *
 * @author favdb
 */
public class EditItem extends AbstractEditor {

	private JComboBox cbCategory;
	private JTextArea relations;

	public EditItem(Editor m, AbstractEntity e) {
		super(m, e, "111");
		initAll();
	}

	@Override
	public void initUpper() {
		Item item = (Item) entity;
		cbCategory = Ui.initAutoCombo(panel, "category",
				Item.findCategories(mainFrame),
				item.getCategory(), !MANDATORY, EMPTY);
		relations = Ui.initRelations(this.panel, mainFrame, entity, this);
	}

	@Override
	public boolean verifier() {
		resetError();
		return (msgError.isEmpty());
	}

	@Override
	public void apply() {
		Item item = (Item) entity;
		item.setCategory((String) cbCategory.getSelectedItem());
		super.apply();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof JButton) {
			JButton bt = (JButton) e.getSource();
			switch (bt.getName()) {
				case "btRelation":
					//App.trace("relationship to modify for " + entity.getName());
					if (RelationsDlg.show(editor)) {
						updateRelations();
					}
					break;
				default:
					break;

			}
		}
	}

	private void updateRelations() {
		List<String> list = new ArrayList<>();
		//zone de liste à plat sur 2 lignes max
		for (Relationship relation : EntityUtil.findRelation(mainFrame, entity)) {
			list.add(relation.getName());
		}
		relations.setText(ListUtil.join(list, ", "));
	}

}
