/*
 * Copyright (C) 2021 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import i18n.I18N;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import storybook.App;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Scene;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSLabel;
import storybook.ui.Ui;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.panel.typist.TypistScenario;

/**
 *
 * @author favdb
 */
public class EditScenario extends AbstractPanel {

	private JComboBox cbMoment,
			cbLocation,
			cbStart,
			cbEnd;
	private JTextField tfPitch;
	private JRadioButton rbLocInt, rbLocExt;
	private JButton btLoc;
	private TypistScenario scenarioPanel;
	private EditScene panelScene;

	public EditScenario(TypistScenario scenarioPanel) {
		super(scenarioPanel.mainFrame);
		this.scenarioPanel = scenarioPanel;
		initAll();
	}

	public EditScenario(EditScene panelScene) {
		super(panelScene.mainFrame);
		this.panelScene = panelScene;
		initAll();
	}

	@Override
	public void init() {
		// empty
	}

	@Override
	public void initUi() {
		this.setLayout(new MigLayout("hidemode 3"));
		// pitch
		add(new JSLabel(I18N.getColonMsg("scenario.pitch")), "span, split 2");
		tfPitch = new JTextField();
		tfPitch.setName("tfPitch");
		tfPitch.setFont(App.getInstance().fontGetEditor());
		add(tfPitch, "growx");
		// liste des moments
		add(new JSLabel(I18N.getColonMsg("scenario.moment")), "al right");
		cbMoment = initCbScenario("moment");
		add(cbMoment);
		// radio des INT/EXT
		add(new JSLabel(I18N.getColonMsg("location")));
		rbLocInt = new JRadioButton(I18N.getMsg("scenario.loc.int"));
		rbLocInt.setName("rbInt");
		add(rbLocInt);
		rbLocExt = new JRadioButton(I18N.getMsg("scenario.loc.ext"));
		rbLocExt.setName("rbExt");
		ButtonGroup group = new ButtonGroup();
		group.add(rbLocInt);
		group.add(rbLocExt);
		add(rbLocExt);
		// liste des lieux
		cbLocation = new JComboBox();
		cbLocation.setName("cbLocation");
		Location location = null;
		if (panelScene != null) {
			Scene scene = (Scene) panelScene.entity;
			if (scene.getLocations() != null && !scene.getLocations().isEmpty()) {
				location = scene.getLocations().get(0);
			}
		}
		EntityUtil.fillCB(mainFrame, cbLocation, Book.TYPE.LOCATION, location, false, false);
		add(cbLocation);
		btLoc = Ui.initButton("btLoc", "", ICONS.K.NEW, "location.add", this);
		add(btLoc, "wrap");
		// liste de la transition du début
		add(new JSLabel(I18N.getColonMsg("scenario.start")));
		cbStart = initCbScenario("in");
		add(cbStart);
		// liste de la transition de fin
		add(new JSLabel(I18N.getColonMsg("scenario.end")), "span, split 2, right");
		cbEnd = initCbScenario("out");
		add(cbEnd);
	}

	@SuppressWarnings("unchecked")
	public JComboBox initCbScenario(String name) {
		//App.trace("EditScenario.initCbScenario("+action.toString()+")");
		JComboBox cb = new JComboBox();
		cb.setName(name);
		for (int i = 0;; i++) {
			String s = findScenarioKey(name, i);
			if (s.isEmpty()) {
				break;
			}
			cb.addItem(s);
		}
		return (cb);
	}

	public String findScenarioKey(String type, int i) {
		//App.trace("EditScenario.findKey(type=" + type + ", i=" + i + ")");
		String key = "scenario." + type + "." + String.format("%02d", i);
		try {
			String msg = I18N.getMsg(key);
			if (msg.equals("!" + key + "!")) {
				return ("");
			}
			return (msg);
		} catch (Exception ex) {
			return ("");
		}
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent evt) {
		boolean toRefresh = false;
		if (evt.getSource() instanceof JButton) {
			JButton bt = (JButton) evt.getSource();
			switch (bt.getName()) {
				case "btLoc":
					Location entity = new Location();
					JDialog dlg = new JDialog(mainFrame.fullFrame, true);
					Editor editor = new Editor(mainFrame, entity, dlg);
					dlg.setTitle(I18N.getMsg("editor"));
					dlg.add(editor);
					Dimension pos = SwingUtil.getDlgPosition(entity);
					Dimension size = SwingUtil.getDlgSize(entity);
					dlg.setSize((size != null ? size.height : this.getWidth() / 2),
							(size != null ? size.width : 680));
					if (pos != null) {
						dlg.setLocation(pos.height, pos.width);
					} else {
						dlg.setLocationRelativeTo(this);
					}
					dlg.setVisible(true);
					SwingUtil.saveDlgPosition(dlg, entity);
					if (!editor.canceled) {
						EntityUtil.fillCB(mainFrame, cbLocation, Book.TYPE.LOCATION, null, false, false);
						cbLocation.setSelectedItem(cbLocation.getItemCount() - 1);
						mainFrame.setUpdated();
					}
					break;
				default:
					break;
			}
		} else if (evt.getSource() instanceof JComboBox) {
			JComboBox cb = (JComboBox) evt.getSource();
			switch (cb.getName()) {
				case "cbLocations":
				case "cbLoc":
				case "cbMoment":
				case "cbIn":
				case "cbOut":
				default:
					toRefresh = true;
					break;
			}
		} else if (evt.getSource() instanceof JRadioButton) {
			toRefresh = true;
		}
		if (toRefresh) {
			if (scenarioPanel != null) {
				Scene scene = scenarioPanel.getScenarioData();
				scenarioPanel.getMdEdit().setHeader(scene, book.info.scenarioGet());
				scenarioPanel.setModified();
			} else if (panelScene != null) {
				getScenarioData((Scene) panelScene.entity);
			}
		}
	}

	public String getPitch() {
		return tfPitch.getText();
	}

	public void getScenarioData(Scene scene) {
		//App.trace("EditScenario.getScenarioData()");
		//Scene scene = (Scene) panelScene.entity;
		//scene.setScenario_stage(cbStage.getSelectedIndex());
		List<Location> locs = new ArrayList<>();
		if (cbLocation.getSelectedItem() != null) {
			locs.add((Location) cbLocation.getSelectedItem());
		}
		scene.setLocations(locs);
		scene.setScenario_pitch(tfPitch.getText());
		scene.setScenario_moment(cbMoment.getSelectedIndex());
		scene.setScenario_loc((rbLocInt.isSelected() ? 1 : 2));
		scene.setScenario_start(cbStart.getSelectedIndex());
		scene.setScenario_end(cbEnd.getSelectedIndex());
		if (panelScene.getMkTexte() != null) {
			panelScene.getMkTexte().setHeader(scene, book.info.scenarioGet());
		}
		panelScene.modified = true;
	}

	public void setScene(Scene scene) {
		//cbStage.setSelectedIndex(scene.getScenario_stage());
		tfPitch.setText(scene.getScenario_pitch());
		cbMoment.setSelectedIndex(scene.getScenario_moment());
		cbStart.setSelectedIndex(scene.getScenario_start());
		cbEnd.setSelectedIndex(scene.getScenario_end());
		if (scene.getScenario_loc() == 1) {
			rbLocInt.setSelected(true);
		}
		if (scene.getScenario_loc() == 2) {
			rbLocExt.setSelected(true);
		}
	}

}
