/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import javax.swing.JOptionPane;
import static storybook.exim.exporter.AbstractExport.SILENT;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Episode;
import storybook.model.hbn.entity.Strand;
import storybook.tools.DateUtil;
import storybook.tools.html.Html;
import storybook.ui.MainFrame;

/**
 * export the Episode content to HTML format
 *
 * @author favdb
 */
public class ExportEpisodeToHtml extends AbstractExport {

	private static final String TT = "ExportEpisode";
	private List<Episode> episodes;
	private List<Strand> strands;

	public ExportEpisodeToHtml(MainFrame m) {
		super(m, "html");
		init();
	}

	@SuppressWarnings("unchecked")
	private void init() {
		//LOG.trace(TT + ".init()");
		episodes = EntityUtil.findEntities(mainFrame, Book.TYPE.EPISODE);
	}

	/**
	 * do an export to a file
	 *
	 * @param m
	 * @return true if export to file is ok
	 */
	public static boolean toFile(MainFrame m) {
		//LOG.trace(TT + ".toFile(mainFrame)");
		ExportEpisodeToHtml export = new ExportEpisodeToHtml(m);
		return export.writeFile();
	}

	/**
	 * write a write
	 *
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean writeFile() {
		//LOG.trace(TT + ".writeFile()");
		String fileName = "Episode";
		if (askFileExists(fileName)) {
			Object[] options = {
				I18N.getMsg("replace"),
				I18N.getMsg("export.replace_add_date"),
				I18N.getMsg("cancel"),};
			int r = JOptionPane.showOptionDialog(mainFrame,
					I18N.getMsg("export.replacenew", param.getFileName()),
					I18N.getMsg("export"),
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);
			switch (r) {
				case 0:
					break;
				case 1:
					SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
					fileName = fileName + formatter.format(new Date());
					break;
				case 2:
					return (false);
			}
		}
		setHtmlTitle(I18N.getColonMsg("episodes") + " " + book.getTitle());
		if (!openFile(fileName)) {
			return (false);
		}
		String title = I18N.getColonMsg("episodes") + " " + book.getTitle();
		writeText(Html.intoH(1, title));
		writeText(Html.P_CENTER + Html.intoI("(" + DateUtil.simpleDateTimeToString(new Date(), true) + ")") + Html.P_E);
		writeText(Html.TABLE_B);
		writeText(Html.TR_B);
		writeText(Html.intoTD(I18N.getMsg("number")));
		writeText(Html.intoTD(I18N.getMsg("name")));
		writeText(Html.intoTD(I18N.getMsg("chapters") + "/" + I18N.getMsg("scenes")));
		writeText(Html.intoTD(I18N.getMsg("plot")));
		strands = EntityUtil.findEntities(mainFrame, Book.TYPE.STRAND);
		for (Strand strand : strands) {
			writeText(Html.intoTD(strand.getName()));
		}
		writeText(Html.TR_E);
		getEpisodes();
		writeText(Html.TABLE_E);
		closeFile(SILENT);
		JOptionPane.showMessageDialog(mainFrame,
				I18N.getMsg("export.success", param.getFileName()),
				I18N.getMsg("export"), JOptionPane.INFORMATION_MESSAGE);
		return true;
	}

	private void getEpisodes() {
		boolean b = false;
		int col = 0;
		for (int i = 0; i < episodes.size(); i++) {
			Episode episode = episodes.get(i);
			if (episode.getStrand() == null) {
				if (b) {
					while (col < strands.size()) {
						writeText(Html.intoTD("&nbsp;"));
						col++;
					}
					writeText(Html.TR_E);
				}
				writeText(Html.TR_B);
				writeText(Html.intoTD(episode.getNumber().toString(), "style=\"vertical-align: top;text-align: center;\""));
				writeText(Html.intoTD(episode.getName(), "style=\"vertical-align: top;\""));
				if (episode.hasLink()) {
					writeText(Html.intoTD(episode.getLink().getName(), "style=\"vertical-align: top;text-align: center;\""));
				} else {
					writeText(Html.intoTD("&nbsp;"));
				}
				String txt = episode.getNotes();
				if (txt.isEmpty()) {
					writeText(Html.intoTD("&nbsp;"));
				} else {
					writeText(Html.intoTD(txt, "style=\"vertical-align: top;\""));
				}
				b = true;
				col = 0;
			} else {
				int ncol = strands.indexOf(episode.getStrand());
				for (int j = col; j < ncol; j++) {
					writeText(Html.intoTD("&nbsp;"));
					col++;
				}
				String txt = episode.getNotes();
				if (txt.isEmpty()) {
					writeText(Html.intoTD("&nbsp;"));
				} else {
					writeText(Html.intoTD(txt, "style=\"vertical-align: top;\""));
				}
				col++;
			}
		}
		if (b) {
			writeText(Html.TR_E);
		}
	}

}
