/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter.options;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.miginfocom.swing.MigLayout;
import storybook.ui.Ui;
import storybook.exim.exporter.ExportBookDlg;
import i18n.I18N;
import storybook.model.book.BookParamExport;
import storybook.tools.swing.js.JSLabel;
import storybook.ui.MIG;

/**
 *
 * @author favdb
 */
public class BOOKpanel extends JPanel implements ChangeListener {

	private static final String TT = "BOOKPanel";

	private final BookParamExport paramExport;
	private JCheckBox ckChapterBooktitle;
	private JRadioButton rbMultiChapter;
	private JRadioButton rbMultiScene;
	public JRadioButton rbOneFile;
	private JRadioButton rbIgnore;
	private JRadioButton rbSuppress;
	private JRadioButton rbLeft;
	private final ExportBookDlg dlgExport;

	public BOOKpanel(ExportBookDlg dlg) {
		this.dlgExport = dlg;
		this.paramExport = dlg.getMainFrame().getBook().getParamExport();
		initUi();
	}

	private void initUi() {
		setLayout(new MigLayout(MIG.get(MIG.HIDEMODE2, MIG.WRAP), "[][]"));
		add(new JLabel(I18N.getMsg("export.book.htmloption")), MIG.SPAN);
		ButtonGroup bg1 = new ButtonGroup();
		rbOneFile = new JRadioButton();
		rbOneFile.setText(I18N.getMsg("export.book.htmloption.onefile"));
		bg1.add(rbOneFile);
		add(rbOneFile, MIG.SKIP + " 1");

		rbMultiChapter = new JRadioButton();
		rbMultiChapter.setText(I18N.getMsg("export.book.htmloption.multichapter"));
		bg1.add(rbMultiChapter);
		add(rbMultiChapter, MIG.get(MIG.SKIP + " 1", MIG.SPLIT2));

		ckChapterBooktitle = new JCheckBox(I18N.getMsg("export.chapter.booktitle"));
		ckChapterBooktitle.setSelected(paramExport.getChapterBookTitle());
		add(ckChapterBooktitle);

		rbMultiScene = new JRadioButton();
		rbMultiScene.setText(I18N.getMsg("export.book.htmloption.multiscene"));
		bg1.add(rbMultiScene);
		add(rbMultiScene, MIG.SKIP + " 1");

		rbOneFile.addChangeListener(this);
		rbMultiChapter.addChangeListener(this);
		rbMultiScene.addChangeListener(this);

		rbOneFile.setSelected(true);
		rbMultiChapter.setSelected(paramExport.getHtmlMultiChapter());
		rbMultiScene.setSelected(paramExport.getHtmlMultiScene());

		ckChapterBooktitle.setEnabled(rbMultiChapter.isSelected());
		add(new JSLabel(I18N.getMsg("export.book.highlight")), MIG.SPAN);
		int t = paramExport.getHighlight();
		ButtonGroup bg2 = new ButtonGroup();
		rbIgnore = Ui.initRadio(bg2, "ignore", "export.book.highlight.ignore", t == 0);
		add(rbIgnore, MIG.SKIP + " 1");
		rbSuppress = Ui.initRadio(bg2, "suppress", "export.book.highlight.suppress", t == 1);
		add(rbSuppress, MIG.SKIP + " 1");
		rbLeft = Ui.initRadio(bg2, "left", "export.book.highlight.left", t == 2);
		add(rbLeft, MIG.SKIP + " 1");
	}

	public void apply() {
		//App.trace(TT + ".apply()");
		paramExport.setChapterBookTitle(ckChapterBooktitle.isSelected());
		paramExport.setHtmlMultiChapter(rbMultiChapter.isSelected());
		paramExport.setHtmlMultiScene(rbMultiScene.isSelected());
		int x = 0;
		if (rbSuppress.isSelected()) {
			x = 1;
		}
		if (rbLeft.isSelected()) {
			x = 2;
		}
		paramExport.setHighlight(x);
		paramExport.save();
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		dlgExport.setOneFile(!rbOneFile.isSelected());
		ckChapterBooktitle.setEnabled(rbMultiChapter.isSelected() || rbMultiScene.isSelected());
	}

}
