/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.help;

import i18n.I18N;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import org.miginfocom.swing.MigLayout;
import storybook.tools.html.Html;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.dialog.AbstractDialog;

/**
 *
 * @author favdb
 */
public class Shortcut extends AbstractDialog {

	private static final String TT = "Shortcut";

	private final String SHIFT[] = {
		"char_unbreak", "line_break"
	};
	private final String CTRL[] = {
		"bold", "italic", "strike", "underline", "superscript",
		"copy", "cut", "paste",
		"undo", "redo", "zoom.in", "zoom.out"
	};
	private final String CTRL_SHIFT[] = {
		"al_left", "al_center", "al_right", "al_justify"
	};
	private final String CTRL_ALT[] = {};
	private final String ALT[] = {
		"listordered", "listunordered",
		"image", "hyperlink_", "table_",
		"char_cadratin",
		"endnote.add", "endnotes.show"
	};
	private final String ALT_SHIFT[] = {};

	public static void show(MainFrame mainFrame) {
		Shortcut dlg = new Shortcut(mainFrame);
		dlg.setVisible(true);
	}

	public Shortcut(MainFrame mainFrame) {
		super(mainFrame);
		initAll();
	}

	@Override
	public void init() {
		//empty
	}

	@Override
	public void initUi() {
		this.setLayout(new MigLayout(MIG.get(MIG.FLOWX, MIG.WRAP), "[grow]"));
		this.setTitle(I18N.getMsg("help.shortcut"));

		JEditorPane ta = initText();
		JScrollPane scroller = new JScrollPane(ta);
		scroller.setPreferredSize(new Dimension(640, 480));
		add(scroller, MIG.get(MIG.SPAN, MIG.GROW));

		add(getCloseButton(), MIG.get(MIG.SPAN, MIG.RIGHT));
		this.pack();
		this.setLocationRelativeTo(mainFrame);
		this.setModal(true);
	}

	private JEditorPane initText() {
		JEditorPane ed = new JEditorPane();
		ed.setContentType("text/html");
		ed.setEditable(false);
		StringBuilder buf = new StringBuilder();
		buf.append(Html.HTML_B);
		buf.append(Html.BODY_B);
		buf.append(Html.TABLE_B);
		buf.append(getShortcutSimple("F7", "jortho.spelling"));
		String shift = I18N.getMsg("shef.shift");
		String ctrl = I18N.getMsg("shef.ctrl");
		String alt = I18N.getMsg("shef.alt");
		buf.append(getShortcut(shift, SHIFT));
		buf.append(getShortcut(ctrl, CTRL));
		buf.append(getShortcut(ctrl + "+" + shift, CTRL_SHIFT));
		buf.append(getShortcut(ctrl + "+" + alt, CTRL_ALT));
		buf.append(getShortcut(alt, ALT));
		buf.append(getShortcut(alt + "+" + shift, ALT_SHIFT));
		buf.append(Html.TABLE_E);
		buf.append(Html.BODY_E);
		buf.append(Html.HTML_E);
		ed.setText(buf.toString());
		ed.setCaretPosition(0);
		return ed;
	}

	private String getShortcut(String modifier, String shortcuts[]) {
		StringBuilder buf = new StringBuilder();
		for (String shef : shortcuts) {
			String sh = I18N.getMsg("shef." + shef + ".mnemonic");
			if (!sh.startsWith("!")) {
				buf.append(Html.TR_B);
				buf.append(Html.intoTD(modifier + "+" + sh));
				buf.append(Html.intoTD(I18N.getMsg("shef." + shef)));
				buf.append(Html.TR_E);
			}
		}
		return buf.toString();
	}

	private String getShortcutSimple(String key, String shortcut) {
		StringBuilder buf = new StringBuilder();
		buf.append(Html.TR_B);
		buf.append(Html.intoTD(key));
		buf.append(Html.intoTD(I18N.getMsg(shortcut)));
		buf.append(Html.TR_E);
		return buf.toString();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		//empty
	}
}
