/*
 * Copyright (C) 2018 FaVdB
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a fileCopy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.tools.file;

import i18n.I18N;
import java.io.File;
import javax.swing.JOptionPane;
import org.w3c.dom.Element;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Scene;
import storybook.ui.MainFrame;

/**
 *
 * @author FaVdB
 */
public class TempUtil {

	public static AbstractEntity check(MainFrame mainFrame, AbstractEntity entity, boolean ignore) {
		if (!ignore && entity instanceof Scene) {
			Scene scene = (Scene) entity;
			File file = getFile(mainFrame, entity);
			if (file.exists()) {
				String saved = IOUtil.fileReadAsString(file.getAbsolutePath());
				if (!saved.equals(scene.toXml())) {
					int rc = JOptionPane.showConfirmDialog(null, I18N.getMsg("file.backup_askrestore"),
							I18N.getMsg("file.backup_rest"), JOptionPane.YES_NO_OPTION);
					if (rc == JOptionPane.YES_OPTION) {
						StringBuilder str = new StringBuilder();
						str.append("<?xml version='1.0'?>\n");
						str.append("<book xmlns=\"http://docbook.org/ns/docbook\" version=\"5.0\">\n");
						str.append(saved);
						str.append("</book>\n");
						Element rootNode = Xml.getRootNode(str.toString());
						if (rootNode == null) {
							return (scene);
						}
						Element n = (Element) rootNode.getElementsByTagName("scene").item(0);
						if (n != null) {
							scene = Scene.fromXml(n);
						}
					}
				}
			}
			write(mainFrame, entity);
			return (scene);
		}
		return (entity);
	}

	private static File getFile(MainFrame mainFrame, AbstractEntity entity) {
		Scene scene = (Scene) entity;
		String ident = "scene_" + scene.getId() + ".tmp";
		File f = new File(mainFrame.getH2File().getPath() + File.separator + ident);
		return (f);
	}

	public static void remove(MainFrame mainFrame, AbstractEntity entity, boolean force) {
		if (entity instanceof Scene) {
			if (getFile(mainFrame, entity).exists()) {
				getFile(mainFrame, entity).delete();
			}
		}
	}

	public static void write(MainFrame mainFrame, AbstractEntity entity) {
		if (entity instanceof Scene) {
			Scene scene = (Scene) entity;
			IOUtil.fileWriteString(getFile(mainFrame, entity), scene.toXml());
		}
	}

}
