/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.dialog;

import i18n.I18N;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.ChapterDAO;
import storybook.model.hbn.dao.IdeaDAO;
import storybook.model.hbn.dao.ItemDAO;
import storybook.model.hbn.dao.LocationDAO;
import storybook.model.hbn.dao.MemoDAO;
import storybook.model.hbn.dao.PartDAO;
import storybook.model.hbn.dao.PersonDAO;
import storybook.model.hbn.dao.PlotDAO;
import storybook.model.hbn.dao.SceneDAO;
import storybook.model.hbn.dao.StrandDAO;
import storybook.model.hbn.dao.TagDAO;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.tools.TextUtil;
import storybook.tools.html.Html;
import storybook.tools.swing.FontUtil;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSLabel;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import static storybook.ui.Ui.MINIMUM_SIZE;

/**
 *
 * @author favdb
 */
public class SearchDlg extends AbstractDialog {

	private String[] words;
	private JPanel resultat = new JPanel();
	private JTextField txWords;
	List<JCheckBox> cbList;
	String[] objects = {"strand", "part", "chapter", "scene", "person", "location", "item", "tag", "idea", "memo", "plot"};
	private JButton btAll;
	private JButton btFind;
	private int nbResultat;
	private JLabel lbResultat;

	public SearchDlg(MainFrame m) {
		super(m);
		initAll();
	}

	@Override
	public void init() {
		// empty
	}

	@Override
	public void initUi() {
		JSLabel jLabel1 = new JSLabel(I18N.getMsg("search.words"));
		txWords = new JTextField(32);
		btAll = new JButton(I18N.getMsg("all"));
		btAll.addActionListener(evt -> selectAll());
		JPanel jPanel1 = new JPanel(new MigLayout("wrap 4"));
		jPanel1.setBorder(BorderFactory.createTitledBorder(I18N.getMsg("search.for")));
		cbList = new ArrayList<>();
		for (String str : objects) {
			JCheckBox cb = new JCheckBox(I18N.getMsg(str));
			cb.setName(str);
			cb.addActionListener(evt -> checkIfOk());
			jPanel1.add(cb);
			cbList.add(cb);
		}
		jPanel1.add(new JSLabel(""), MIG.WRAP);
		jPanel1.add(btAll, MIG.get(MIG.SPAN, MIG.RIGHT));

		btFind = new JButton(I18N.getMsg("find"));
		btFind.setIcon(IconUtil.getIconSmall(ICONS.K.SEARCH));
		btFind.setEnabled(false);
		btFind.addActionListener((ActionEvent evt) -> {
			searchEntities();
		});
		//layout
		setLayout(new MigLayout());
		setTitle(I18N.getMsg("search"));
		add(jLabel1, MIG.WRAP);
		add(txWords, MIG.get(MIG.CENTER, MIG.WRAP));
		add(jPanel1, MIG.get(MIG.SPAN, MIG.WRAP));
		add(getCancelButton(), MIG.get(MIG.SG, MIG.SPAN, MIG.SPLIT2, MIG.RIGHT));
		add(btFind, MIG.SG);
		pack();
		setLocationRelativeTo(mainFrame);
		this.setModal(true);
	}

	private void checkIfOk() {
		boolean b = (!txWords.getText().isEmpty());
		boolean bb = false;
		for (JCheckBox cb : cbList) {
			if (cb.isSelected()) {
				bb = true;
				break;
			}
		}
		if (bb == false) {
			b = false;
		}
		btFind.setEnabled(b);
	}

	private void selectAll() {
		boolean x;
		if (btAll.getText().equals(I18N.getMsg("all"))) {
			btAll.setText(I18N.getMsg("none"));
			x = true;
		} else {
			btAll.setText(I18N.getMsg("all"));
			x = false;
		}
		for (JCheckBox cb : cbList) {
			cb.setSelected(x);
		}
		checkIfOk();
	}

	private void searchEntities() {
		String s = Html.htmlToText(txWords.getText());
		if (s.isEmpty()) {
			return;
		}
		words = s.split(" ");
		for (String word : words) {
			if (word.length() < 4) {
				SwingUtil.showError("search.error.word");
				return;
			}
		}
		initResultat();
		boolean x = false;
		for (JCheckBox cb : cbList) {
			if (cb.isSelected()) {
				switch (Book.getTYPE(cb.getName())) {
					case STRAND:
						findStrands();
						break;
					case PART:
						findParts();
						break;
					case CHAPTER:
						findChapters();
						break;
					case SCENE:
						findScenes();
						break;
					case PERSON:
						findPersons();
						break;
					case PLOT:
						findPlots();
						break;
					case LOCATION:
						findLocations();
						break;
					case ITEM:
						findItems();
						break;
					case TAG:
						findTags();
						break;
					case IDEA:
						findIdeas();
						break;
					case MEMO:
						findMemos();
						break;
					default:
						break;
				}
				x = true;
			}
		}
		if (!x) {
			SwingUtil.showError("search.error.object");
			return;
		}
		if (nbResultat > 0) {
			lbResultat.setText(I18N.getMsg("search.find", nbResultat));
		}
		showResults(resultat);
	}

	private void findStrands() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		StrandDAO dao = new StrandDAO(session);
		List<Strand> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("strand"));
		for (Strand entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findParts() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAO dao = new PartDAO(session);
		List<Part> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("part"));
		for (Part entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findChapters() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ChapterDAO dao = new ChapterDAO(session);
		List<Chapter> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("chapter"));
		for (Chapter entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getChapternoStr() + " " + entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findScenes() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("scene"));
		for (Scene entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))
					|| searchWordsHtml(entity.getSummary())) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getFullTitle(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findPersons() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PersonDAO dao = new PersonDAO(session);
		List<Person> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("person"));
		for (Person entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getFullNameAbbr(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findPlots() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PlotDAO dao = new PlotDAO(session);
		List<Plot> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("plot"));
		for (Plot entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findLocations() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		List<Location> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("location"));
		for (Location entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findItems() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemDAO dao = new ItemDAO(session);
		List<Item> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("item"));
		for (Item entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findTags() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TagDAO dao = new TagDAO(session);
		List<Tag> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("tag"));
		for (Tag entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findIdeas() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		IdeaDAO dao = new IdeaDAO(session);
		List<Idea> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("idea"));
		for (Idea entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getId() + " " + TextUtil.ellipsize(entity.getNotes(), 30), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private void findMemos() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		MemoDAO dao = new MemoDAO(session);
		List<Memo> entities = dao.findAll();
		boolean finds = false;
		doTitle(I18N.getMsg("memo"));
		for (Memo entity : entities) {
			if (searchWords(entity.toCsv(" ", " ", "\t"))) {
				if (!finds) {
					doNext();
				}
				doEntity(entity.getName(), entity);
				finds = true;
			}
		}
		if (!finds) {
			doEmpty();
		}
	}

	private boolean searchWords(String str) {
		String r = Html.htmlToText(str);
		for (String word : words) {
			if (r.toLowerCase(Locale.getDefault()).contains(word.toLowerCase(Locale.getDefault()))) {
				return (true);
			}
		}
		return (false);
	}

	private boolean searchWordsHtml(String str) {
		String r = Html.htmlToText(str);
		for (String word : words) {
			if (r.toLowerCase(Locale.getDefault()).contains(word.toLowerCase(Locale.getDefault()))) {
				return (true);
			}
		}
		return (false);
	}

	private void showResults(JPanel res) {
		SearchResultsDlg dlg = new SearchResultsDlg(mainFrame, res);
		dlg.setVisible(true);
	}

	private void initResultat() {
		resultat.setLayout(new MigLayout(MIG.get(MIG.TOP, MIG.WRAP), "[][]"));
		resultat.setBackground(Color.white);
		resultat.setMinimumSize(MINIMUM_SIZE);
		resultat.removeAll();
		lbResultat = new JLabel("");
		resultat.add(lbResultat, MIG.SPAN);
		nbResultat = 0;
	}

	private void doTitle(String msg) {
		JSLabel lb = new JSLabel(msg);
		lb.setFont(FontUtil.getBold());
		resultat.add(lb);
	}

	private void doEmpty() {
		JSLabel r = new JSLabel(I18N.getMsg("search.empty"));
		resultat.add(r, MIG.WRAP);
	}

	private void doEntity(String str, AbstractEntity entity) {
		resultat.add(new JLabel(IconUtil.getIconSmall(ICONS.K.NAV_NEXT)), MIG.RIGHT);
		JLabel r = new JLabel(str);
		r.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				mainFrame.showEditorAsDialog(entity);
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				resultat.setCursor(new Cursor(Cursor.HAND_CURSOR));
			}

			@Override
			public void mouseExited(MouseEvent e) {
				resultat.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		});
		resultat.add(r, MIG.WRAP);
		nbResultat++;
	}

	private void doNext() {
		JLabel r = new JLabel(" ");
		resultat.add(r, MIG.WRAP);
	}

	public static void show(MainFrame m) {
		SwingUtil.showModalDialog(new SearchDlg(m), m, true);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

}
