/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.dialog.chooser;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import storybook.tools.swing.ColorIcon;
import storybook.tools.swing.ColorUtil;
import storybook.tools.swing.ColorUtil.PALETTE;
import storybook.tools.swing.FontUtil;

/**
 *
 * @author favdb
 */
public class ColorPicker extends JComboBox {

	@SuppressWarnings("unchecked")
	public ColorPicker(Color color) {
		super();
		this.setRenderer(new LCRColor());
		for (PALETTE c : PALETTE.values()) {
			this.addItem(c.getColor());
		}
		setSelectedItem(color);
	}

	private static class LCRColor extends DefaultListCellRenderer {

		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index,
				boolean sel, boolean focus) {
			JLabel lb = (JLabel) super.getListCellRendererComponent(list, value, index, sel, focus);
			if (value instanceof Color) {
				Color color = (Color) value;
				lb.setIcon(new ColorIcon(color,
						new Dimension(FontUtil.getWidth() * 8, FontUtil.getHeight())));
				lb.setToolTipText(ColorUtil.getPaletteString(color));
				lb.setText("");
			} else {
				if (value instanceof JLabel) {
					return (JComponent) value;
				}
			}
			return lb;
		}
	}

}
