/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.typist;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import org.miginfocom.swing.MigLayout;
import shef.ShefEditor;
import static storybook.ui.Ui.MINIMUM_SIZE;
import i18n.I18N;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import storybook.tools.swing.SwingUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.dialog.AbstractDialog;

public class TypistEditText extends AbstractDialog {

    private String text;
    private JLabel lbMessage;
    private static final int MAXLEN = 32768;
    private ShefEditor taText;

    public TypistEditText(MainFrame mainFrame, String msg, String text) {
	super(mainFrame);
	setTitle(I18N.getMsg(msg));
	initAll();
	setText(text);
    }

    @Override
    public void init() {
	// empty
    }

    @Override
    public void initUi() {
	setModal(true);
	setIconImage(IconUtil.getIconImageSmall("edit"));
	setLayout(new MigLayout(MIG.get(MIG.WRAP, MIG.FILL)));
	taText = new ShefEditor("disallow, reduced", "");
	taText.setName("taText");
	JScrollPane scroller = new JScrollPane(taText);
	scroller.setMinimumSize(MINIMUM_SIZE);
	scroller.setMaximumSize(new Dimension(1024, 780));
	SwingUtil.setMaxPreferredSize(scroller);
	add(scroller);
	addOkCancel();
	pack();
	setLocationRelativeTo(this.getParent());
    }

    public void setText(String text) {
	taText.setText(text);
    }

    public String getText() {
	return taText.getText();
    }

    @Override
    public void addOkCancel() {
	add(getOkButton(), MIG.get(MIG.RIGHT, MIG.SPLIT2));
	add(getCancelButton());
    }

    @Override
    protected JButton getOkButton() {
	AbstractAction act = new AbstractAction() {
	    @Override
	    public void actionPerformed(ActionEvent e) {
		canceled = false;
		dispose();
	    }
	};
	JButton bt = new JButton(act);
	bt.setText(I18N.getMsg("ok"));
	bt.setIcon(IconUtil.getIconSmall(ICONS.K.OK));
	SwingUtil.addEnterAction(bt, act);
	return (bt);
    }

    @Override
    protected JButton getCancelButton() {
	AbstractAction act = new AbstractAction() {
	    @Override
	    public void actionPerformed(ActionEvent e) {
		canceled = true;
		dispose();
	    }
	};
	JButton bt = new JButton(act);
	bt.setText(I18N.getMsg("cancel"));
	bt.setIcon(IconUtil.getIconSmall(ICONS.K.CANCEL));
	SwingUtil.addEnterAction(bt, act);
	return (bt);
    }

    @Override
    public boolean isCanceled() {
	return (canceled);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
	// empty
    }

}
