/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.typist;

import i18n.I18N;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import storybook.App;
import storybook.Const;
import storybook.Pref;
import storybook.edit.Editor;
import storybook.edit.Intensity;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.SceneDAO;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.tools.LOG;
import storybook.tools.ShefUtil;
import storybook.tools.StringUtil;
import storybook.tools.file.TempUtil;
import storybook.tools.swing.ColorIcon;
import storybook.tools.swing.ColorLabel;
import storybook.tools.swing.SwingUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.combobox.ComboModelSceneState;
import storybook.ui.dialog.ChapterSelectDlg;
import storybook.ui.dialog.EntitiesListDlg;
import storybook.ui.dialog.ListEndnotesDlg;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.renderer.lcr.LCRSceneState;
import storybook.ui.review.ReviewPanel;

public class TypistPanel extends AbstractPanel {

	private static final String TT = "TypistPanel";

	private static final String CHAPTER = "chapter",
			BT_EXIT = "btExit", BT_IDEA = "btIdea", BT_IGNORE = "btIgnore",
			BT_FIRST = "btFirst", BT_LAST = "btLast",
			BT_PRIOR = "btPrior", BT_NEXT = "btNext",
			BT_NEW = "btNew", BT_SAVE = "btSave",
			BT_STRAND = "btStrand", STRAND_NEW = "strand.new",
			BT_INFO = "btInfo",
			BT_SCREEN = "btScreen", BT_TITLE = "btTitle",
			CB_SCENES = "cbScenes";

	private int origin;
	private Scene scene = null;
	private Dimension screenSize;
	private int defaultHeight, saveSplit;
	private ShefEditor htTexte;
	private TypistInfo panelInfo;
	private JComboBox cbScenes, cbStatus;
	private JButton btNewScene,
			btFirst, btPrior, btNext, btLast,
			btScreen, btChapter, btTitle, btShowInfo,
			btStrand, btHide, btIgnore, btSave;
	private JPanel statusbar;
	private Intensity intensity;
	private JPanel strandsList, tb11, tb2;
	private boolean toRefresh, hiden = false, modified;
	private JButton btEndnotes;
	private List<Endnote> endnotes;
	private JSplitPane split;
	private JScrollPane scrollComments;
	private ReviewPanel reviewPanel;
	private JCheckBox ckReview;

	public TypistPanel(MainFrame mainFrame) {
		super(mainFrame);
		this.scene = getFirstScene();
		initAll();
	}

	public TypistPanel(MainFrame mainFrame, Scene scene) {
		super(mainFrame);
		if (scene == null) {
			this.scene = getFirstScene();
		} else {
			this.scene = (Scene) TempUtil.check(mainFrame, scene, !Const.IGNORE);
		}
		initAll();
	}

	@Override
	public void init() {
		this.withPart = false;
		this.removeAll();
		toRefresh = false;
		App.preferences.setBoolean(Pref.KEY.TYPIST_USE, true);
		hiden = App.preferences.getBoolean(Pref.KEY.TYPIST_SHOWBAR);
	}

	@Override
	public void initUi() {
		//LOG.trace(TT + ".initUi()");
		setLayout(new MigLayout(MIG.get(MIG.INS1, MIG.HIDEMODE3), "[][]"));
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		defaultHeight = 32;
		add(initToolbar(), MIG.get(MIG.SPAN, MIG.GROWX));
		JPanel left = new JPanel(new MigLayout());
		htTexte = new ShefEditor("lang_all, allow, colored", scene.getSummary());
		ShefUtil.setToFileButton(mainFrame, htTexte);
		ShefUtil.setFromFileButton(mainFrame, htTexte);
		panelInfo = new TypistInfo(this, null);
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, htTexte, panelInfo);
		split.setResizeWeight(0.85);
		split.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
		left.add(split, MIG.get(MIG.SPAN, MIG.GROW));
		initEndnote();
		htTexte.wysEditorGet().getToolBar2().add(btEndnotes);
		add(left, MIG.get(MIG.GROW));

		reviewInit();
		add(reviewPanel, MIG.GROW);

		add(initStatusbar(), MIG.get(MIG.NEWLINE, MIG.SPAN));
		if (!App.preferences.getBoolean(Pref.KEY.TYPIST_SHOWINFO)) {
			infosShowHide();
		}
		setScene(scene);
		reviewPanel.setScene(scene);
		setHiden(hiden);
		reviewPanel.setVisible(ckReview.isSelected());
		modified = false;
		origin = htTexte.getText().hashCode();
	}

	@Override
	@SuppressWarnings("unchecked")
	public JToolBar initToolbar() {
		//LOG.trace(TT + ".initToolbar()");
		super.initToolbar();
		//toolbar = new JSToolBar(true);
		toolbar.setFloatable(false);
		toolbar.setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.INS0, MIG.HIDEMODE3)));
		toolbar.setName("TypistToolbar");
		toolbar.add(initTB1(), MIG.GROWX);
		tb11 = initTB11();
		toolbar.add(tb11, MIG.get(MIG.SPAN, MIG.RIGHT));
		tb2 = initTB2();
		toolbar.add(tb2, MIG.get(MIG.GROWX, MIG.NEWLINE));
		return toolbar;
	}

	private JPanel initTB1() {
		//LOG.trace(TT + ".initTB1()");
		JPanel tb1 = new JPanel(new MigLayout());
		tb1.setOpaque(false);
		btHide = Ui.initButton("btHide", "", ICONS.K.DP_UP,
				"typist.show_bar", evt -> setHiden(!hiden));
		tb1.add(btHide);
		cbScenes = Ui.initComboBox(CB_SCENES, "",
				EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE),
				scene, !EMPTY, !ALL, e -> selectScene());
		cbScenes.setMaximumRowCount(15);
		tb1.add(cbScenes);
		btNewScene = Ui.initButton(BT_NEW, "", ICONS.K.ADD,
				"scene.new", e -> doNewScene());
		tb1.add(btNewScene);
		tb1.add(new JLabel(" "));
		//navigation dans les scènes
		btFirst = Ui.initButton(BT_FIRST, "", ICONS.K.NAV_FIRST,
				"export.nav.first", e -> doFirst());
		tb1.add(btFirst);
		btPrior = Ui.initButton(BT_PRIOR, "", ICONS.K.NAV_PREV,
				"export.nav.prior", e -> doPrior());
		tb1.add(btPrior);
		btNext = Ui.initButton(BT_NEXT, "", ICONS.K.NAV_NEXT,
				"export.nav.next", e -> doNext());
		tb1.add(btNext);
		btLast = Ui.initButton(BT_LAST, "", ICONS.K.NAV_LAST,
				"export.nav.last", e -> doLast());
		tb1.add(btLast);
		tb1.add(new JLabel(" "));
		btIgnore = Ui.initButton(BT_IGNORE, "", ICONS.K.CANCEL,
				"discard.changes", e -> refresh());
		tb1.add(btIgnore);
		btSave = Ui.initButton(BT_SAVE, "", ICONS.K.F_SAVE,
				"file.save", e -> save());
		tb1.add(btSave);
		tb1.add(new JLabel(" "));
		btShowInfo = Ui.initButton(BT_INFO, "", ICONS.K.INFO_HIDE,
				"typist.show_infos", e -> doInfo());
		tb1.add(btShowInfo);
		ckReview = Ui.initCheckBox("cbComment", "comments",
				book.getReview(), e -> changeCkReview());
		tb1.add(ckReview);
		return tb1;
	}

	private JPanel initTB11() {
		//LOG.trace(TT + ".initTB11()");
		JPanel tb = new JPanel(new MigLayout(MIG.INS0));
		tb.setOpaque(false);
		tb.setBorder(BorderFactory.createEmptyBorder());
		// idea
		tb.add(Ui.initButton(BT_IDEA, "", ICONS.K.ENT_IDEA,
				"foi.new", e -> mainFrame.showEditorAsDialog(new Idea())));
		tb.add(new JSeparator(), MIG.GROWX);
		//sortie du mode typist
		btScreen = Ui.initButton(BT_SCREEN, "", ICONS.K.SCREEN_NORMAL,
				"screen.normal", e -> returnToMainFrame());
		tb.add(btScreen, MIG.RIGHT);
		// exit
		tb.add(Ui.initButton(BT_EXIT, "", ICONS.K.EXIT,
				"exit", e -> doExit()));
		return tb;
	}

	@SuppressWarnings("unchecked")
	private JPanel initTB2() {
		//LOG.trace(TT + ".initTB2()");
		JPanel tb = new JPanel(new MigLayout(MIG.get(MIG.INS0)));
		tb.setOpaque(false);
		//titre du chapitre
		tb.add(initLabel(CHAPTER, true));
		btChapter = Ui.initButton(CHAPTER, CHAPTER, ICONS.K.ENT_CHAPTER,
				"change", e -> chapterChange());
		tb.add(btChapter);
		//titre de la scène
		tb.add(initLabel("scene", true));
		btTitle = Ui.initButton(BT_TITLE, "", ICONS.K.ENT_SCENE,
				"change", e -> changeSceneTitle());
		tb.add(btTitle);
		cbStatus = new JComboBox(new ComboModelSceneState(false));
		cbStatus.setName(TypistScenario.CMD.CB_STATUS.toString());
		cbStatus.setSelectedIndex(scene.getStatus());
		cbStatus.setRenderer(new LCRSceneState());
		tb.add(initLabel("status", true));
		tb.add(cbStatus);
		intensity = new Intensity(Intensity.LIST, scene.getIntensity());
		tb.add(initLabel("intensity", true));
		tb.add(intensity);
		tb.add(new JLabel(""), MIG.GROWX);
		tb.add(initLabel("strands", true));
		btStrand = new JButton(new ColorIcon(scene.getStrand().getJColor(), IconUtil.getDefSize()));
		btStrand.setName(BT_STRAND);
		btStrand.setToolTipText(scene.getStrand().getName());
		btStrand.addActionListener(this);
		tb.add(btStrand);
		strandsList = new JPanel(new MigLayout(MIG.INS0));
		strandsList.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		strandsList.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				strandsAction();
			}
		});
		strandsRefresh();
		//tb.add(initLabel("strands.other", true));
		tb.add(strandsList);
		return tb;
	}

	private JPanel initStatusbar() {
		//LOG.trace(TT + ".initStatusBar()");
		statusbar = new JPanel();
		statusbar.setName("TypistStatusbar");
		statusbar.setLayout(new MigLayout(MIG.FILLX));
		statusbar.setSize(screenSize.width, defaultHeight);
		return (statusbar);
	}

	public Scene getScene() {
		return this.scene;
	}

	public ShefEditor getHtTexte() {
		return this.htTexte;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		//LOG.trace(TT + ".modelPropertyChange(evt=" + evt.toString() + ")");
	}

	@Override
	@SuppressWarnings("unchecked")
	public void refresh() {
		//LOG.trace(TT + ".refresh()");
		if (scene == null) {
			return;
		}
		if (scene.hasChapter()) {
			btChapter.setText(scene.getChapter().getName());
		} else {
			btChapter.setText(I18N.getMsg("scene.hasnochapter"));
		}
		if (scene.getTitle() == null || scene.getTitle().isEmpty()) {
			btTitle.setText(I18N.getMsg("scene.hasnotitle"));
		} else {
			btTitle.setText(scene.getTitle());
		}
		cbStatus.setSelectedIndex(scene.getStatus());
		intensity.setValue(scene.getIntensity());
		strandRefresh(scene.getStrand());
		strandsRefresh();
		panelInfo.refreshInfo(scene);
		cbScenes.removeActionListener(this);
		cbScenes.setSelectedItem(scene);
		cbScenes.addActionListener(this);
		origin = htTexte.getText().hashCode();
		mainFrame.sceneSetLast(scene);
		refreshText();
		reviewPanel.refresh();
		setNav(EntityUtil.findScenes(mainFrame));
		repaint();
	}

	public void refreshText() {
		//LOG.trace(TT + ".refreshText()");
		if (scene == null) {
			LOG.err("scene is null");
			return;
		}
		scene = EntityUtil.getScene(mainFrame, scene.getId());
		int curpos = htTexte.wysEditorGet().getEditor().getCaretPosition();
		if (curpos < 0) {
			curpos = 0;
		}
		if (scene.getSummary() != null) {
			htTexte.setText(scene.getSummary());
		} else {
			htTexte.setText("");
		}
		if (curpos > htTexte.getText().length()) {
			curpos = 0;
		}
		htTexte.setCaretPosition(curpos);
		origin = htTexte.getText().hashCode();
	}

	public void saveText() {
		scene = EntityUtil.getScene(mainFrame, scene.getId());
		scene.setSummary(htTexte.getText());
		mainFrame.getBookController().updateEntity(scene);
		origin = htTexte.getText().hashCode();
	}

	private void setNav(List<Scene> scenes) {
		if (scenes == null || scenes.isEmpty() || toolbar == null) {
			return;
		}
		btFirst.setEnabled(true);
		btPrior.setEnabled(true);
		btNext.setEnabled(true);
		btLast.setEnabled(true);
		if (scene.equals(scenes.get(0))) {
			btFirst.setEnabled(false);
			btPrior.setEnabled(false);
		} else if (scenes.size() > 1 && scene.equals(scenes.get(scenes.size() - 1))) {
			btNext.setEnabled(false);
			btLast.setEnabled(false);
		}
	}

	private Scene getFirstScene() {
		//LOG.trace(TT + ".getFirstScene()");
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, !Const.FORCE);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findAll();
		if (scenes.isEmpty()) {
			return (null);
		}
		model.commit();
		Scene s = scenes.get(0);
		return (s);
	}

	private Scene getLastScene() {
		//LOG.trace(TT + ".getLastScene()");
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, !Const.FORCE);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findAll();
		if (scenes.isEmpty()) {
			return (null);
		}
		Scene s = scenes.get(scenes.size() - 1);
		return (s);
	}

	private boolean textToLarge(String topText) {
		if (topText.length() > 32767) {
			String str = I18N.getMsg("editor.text_too_large");
			JOptionPane.showMessageDialog(this, str,
					I18N.getMsg("editor"), JOptionPane.ERROR_MESSAGE);
			return (true);
		}
		return (false);
	}

	public int askModified() {
		//LOG.trace(TT + ".askModified()");
		if (scene != null) {
			String topText = htTexte.getText().replace("--", "—&nbsp;");
			int nhash = htTexte.getText().hashCode();
			if (origin != nhash) {
				if (textToLarge(topText)) {
					return (JOptionPane.CANCEL_OPTION);
				}
				setModified();
			}
		}
		if (modified) {
			final Object[] options = {
				I18N.getMsg("save.changes"),
				I18N.getMsg("discard.changes"),
				I18N.getMsg("cancel")
			};
			String str = I18N.getMsg("save.or.discard.changes") + "\n\n"
					+ EntityUtil.getEntityTitle(scene) + ": "
					+ scene.toString() + "\n\n";
			int n = JOptionPane.showOptionDialog(this, str, I18N.getMsg("save.changes.title"),
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, options, options[2]);
			switch (n) {
				case JOptionPane.CANCEL_OPTION:
					return (JOptionPane.CANCEL_OPTION);
				case JOptionPane.YES_OPTION:
					save();
					break;
				case JOptionPane.NO_OPTION:
					break;
				default:
					break;
			}
		}
		return (JOptionPane.YES_OPTION);
	}

	private void newScene() {
		Scene s = new Scene();
		s.setChapter(scene.getChapter());
		JDialog dlg = new JDialog(mainFrame, true);
		dlg.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		Editor editor = new Editor(mainFrame, s, dlg);
		dlg.setTitle(I18N.getMsg("editor"));
		Dimension pos = SwingUtil.getDlgPosition(s);
		dlg.add(editor);
		Dimension size = SwingUtil.getDlgSize(s);
		if (size != null) {
			dlg.setSize(size.height, size.width);
		} else {
			dlg.setSize(this.getWidth() / 2, 680);
		}
		if (pos != null) {
			dlg.setLocation(pos.height, pos.width);
		} else {
			dlg.setLocationRelativeTo(this);
		}
		dlg.setVisible(true);
		if (editor.canceled) {
			return;
		}
		TempUtil.remove(mainFrame, s, Const.FORCE);
		if (editor.entity != null) {
			scene = (Scene) editor.entity;
			refreshCbScenes();
			resetModified();
			cbScenes.setSelectedItem(scene);
		}
		SwingUtil.saveDlgPosition(dlg, s);
		refresh();
	}

	@SuppressWarnings("unchecked")
	private void refreshCbScenes() {
		//LOG.trace(TT + ".refreshCbScenes()");
		cbScenes.removeActionListener(this);
		DefaultComboBoxModel combo = (DefaultComboBoxModel) cbScenes.getModel();
		combo.removeAllElements();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findAll();
		for (Scene entity : scenes) {
			combo.addElement(entity);
		}
		cbScenes.addActionListener(this);
	}

	private void strandChange(String name) {
		//LOG.trace(TT + ".strandChange()");
		String s[] = name.split("_");
		if (s.length > 1 && StringUtil.isNumeric(s[1])) {
			Long id = Long.valueOf(s[1]);
			Strand strand = (Strand) EntityUtil.findEntity(mainFrame, Book.TYPE.STRAND, id);
			if (strand != null) {
				scene.setStrand(strand);
				btStrand.setIcon(new ColorIcon(strand.getJColor(), IconUtil.getDefSize()));
				btStrand.setToolTipText(strand.getName());
				setModified();
			}
		}
	}

	private void chapterChange() {
		//LOG.trace(TT + ".chapterChange()");
		ChapterSelectDlg dlg = new ChapterSelectDlg(mainFrame, scene.getChapter());
		dlg.setVisible(true);
		if (!dlg.isCanceled()) {
			Chapter chapter = dlg.getSelectedChapter();
			scene.setChapter(chapter);
			btChapter.setText(chapter.getName());
			setModified();
		}
	}

	private void changeSceneTitle() {
		LOG.trace(TT + ".changeSceneTitle()");
		String s = (String) JOptionPane.showInputDialog(
				this, "", I18N.getMsg("title"),
				JOptionPane.PLAIN_MESSAGE, null, null, scene.getTitle());
		if ((s != null) && (s.length() > 0)) {
			scene.setTitle(s);
			btTitle.setText(s);
			setModified();
		}
	}

	private boolean save() {
		//LOG.trace(TT + ".save()");
		String topText = htTexte.getText();
		topText = topText.replace("--", "—&nbsp;");
		if (textToLarge(topText)) {
			return (false);
		}
		scene.setSummary(topText);
		scene.setStatus(cbStatus.getSelectedIndex());
		scene.setIntensity(intensity.getValue());
		mainFrame.getBookController().updateEntity(scene);
		if (ckReview.isSelected()) {
			Endnote.renumber(mainFrame, 1);
			reviewPanel.save();
		}
		toRefresh = true;
		resetModified();
		setNav(EntityUtil.findScenes(mainFrame));
		return (true);
	}

	public void setButtons() {
		if (origin != htTexte.getText().hashCode()) {
			modified = true;
		}
		if (toolbar != null) {
			btSave.setEnabled(modified);
			btIgnore.setEnabled(modified);
		}
	}

	public void setModified() {
		modified = true;
		setButtons();
	}

	public void resetModified() {
		modified = false;
		this.origin = htTexte.getText().hashCode();
		setButtons();
	}

	public void closeScene() {
		//reviewPanel.refresh();
		if (modified) {
			save();
		}
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, Const.FORCE);
		}
	}

	private void close() {
		closeScene();
		mainFrame.typistActivate();
	}

	public void tempSave() {
		Scene svScene = new Scene();
		EntityUtil.copyEntityProperties(mainFrame, scene, svScene);
		scene.setSummary(htTexte.getText());
		TempUtil.write(mainFrame, scene);
		EntityUtil.copyEntityProperties(mainFrame, svScene, scene);
	}

	private void doExit() {
		if (!save()) {
			return;
		}
		mainFrame.saveFile(true);
		SwingUtilities.invokeLater(() -> {
			mainFrame.close(true);
		});
	}

	private void doFirst() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		setScene(getFirstScene());
	}

	private void doLast() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		setScene(getLastScene());
	}

	private void doPrior() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		setNextScene(-1);
	}

	private void doNext() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		setNextScene(1);
	}

	private void doNewScene() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		newScene();
	}

	private void doInfo() {
		infosShowHide();
		App.preferences.setBoolean(Pref.KEY.TYPIST_SHOWINFO, panelInfo.isVisible());
		mainFrame.setUpdated();
	}

	private void returnToMainFrame() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		if (toRefresh) {
			SwingUtil.setWaitingCursor(this);
			mainFrame.refresh();
			SwingUtil.setDefaultCursor(this);
		}
		App.preferences.setBoolean(Pref.KEY.TYPIST_USE, false);
		if (modified) {
			mainFrame.setUpdated();
		}
		closeScene();
		mainFrame.typistActivate();
	}

	private void selectScene() {
		if (askModified() == JOptionPane.CANCEL_OPTION) {
			return;
		}
		if (cbScenes.getSelectedItem() != null) {
			Scene s = (Scene) cbScenes.getSelectedItem();
			if (!s.equals(scene)) {
				setScene(s);
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent evt) {
		//LOG.trace(TT + ".actionPerformed(evt=" + evt.toString() + ")");
		JComponent source = (JComponent) evt.getSource();
		if (source.getName().startsWith("strand_")) {
			strandChange(source.getName());
			return;
		}
		switch (source.getName()) {
			case BT_STRAND:
				strandSelect(evt);
				break;
			case STRAND_NEW:
				strandNew();
				break;
			default:
				break;
		}
	}

	private void infosShowHide() {
		//LOG.trace(TT + ".infosShowHide()");
		if (panelInfo.isVisible()) {
			saveSplit = split.getDividerLocation();
		}
		panelInfo.setVisible(!panelInfo.isVisible());
		if (panelInfo.isVisible()) {
			btShowInfo.setIcon(IconUtil.getIconSmall(ICONS.K.INFO_HIDE));
			split.setDividerLocation(saveSplit);
		} else {
			btShowInfo.setIcon(IconUtil.getIconSmall(ICONS.K.INFO));
		}
		split.setEnabled(panelInfo.isVisible());
	}

	private void setScene(Scene s) {
		if (s.equals(scene)) {
			return;
		}
		scene = s;
		cbScenes.removeActionListener(this);
		cbScenes.setSelectedItem(s);
		cbScenes.addActionListener(this);
		refresh();
		reviewPanel.setScene(s);
		htTexte.setCaretPosition(0);
		resetModified();
	}

	private void setNextScene(int i) {
		int n1 = cbScenes.getSelectedIndex() + i;
		if (n1 > -1 && n1 < cbScenes.getItemCount()) {
			cbScenes.setSelectedItem(cbScenes.getItemAt(n1));
		}
	}

	private JMenuItem getMenuItem(String name) {
		JMenuItem m = new JMenuItem(I18N.getMsg(name));
		m.setName(name);
		m.setText(I18N.getMsg(name));
		m.setIcon(IconUtil.getIconSmall(ICONS.K.NEW));
		m.addActionListener(this);
		return m;
	}

	private JMenuItem getMenuItem(Strand strand) {
		JMenuItem m = new JMenuItem(strand.getName());
		m.setName("strand_" + strand.getId().toString());
		m.setIcon(new ColorIcon(strand.getJColor(), IconUtil.getDefSize()));
		m.addActionListener(this);
		return m;
	}

	private void strandsAction() {
		EntitiesListDlg dlg = new EntitiesListDlg(this, mainFrame, scene, "strand");
		dlg.setModal(true);
		dlg.setVisible(true);
		if (!dlg.isCanceled()) {
			scene.setStrands(dlg.getStrands());
			strandsRefresh();
			setModified();
		}
	}

	private void strandsRefresh() {
		strandsList.removeAll();
		if (!scene.getStrands().isEmpty()) {
			for (Strand str : scene.getStrands()) {
				strandsList.add(new ColorLabel(str.getJColor()));
			}
			strandsList.setToolTipText(scene.getStrandsHtml());
		} else {
			strandsList.add(new JLabel("..."));
			strandsList.setToolTipText("");
		}
		strandsList.revalidate();
	}

	private void strandSelect(ActionEvent evt) {
		//LOG.trace(TT + ".strandSelect()");
		JPopupMenu menu = new JPopupMenu();
		JButton bt = (JButton) evt.getSource();
		for (Strand strand : EntityUtil.findStrands(mainFrame)) {
			menu.add(getMenuItem(strand));
		}
		menu.add(new JSeparator());
		JMenuItem addstrand = getMenuItem(STRAND_NEW);
		menu.add(addstrand);
		menu.show(bt, 0, bt.getBounds().height);
	}

	private void strandRefresh(Strand strand) {
		//LOG.trace(TT + ".strandRefresh(" + AbstractEntity.trace(strand) + ")");
		scene.setStrand(strand);
		btStrand.setIcon(new ColorIcon(scene.getStrand().getJColor(), IconUtil.getDefSize()));
		btStrand.setToolTipText(scene.getStrand().getName());
	}

	private void strandNew() {
		//LOG.trace(TT + ".strandNew()");
		Strand strand = new Strand();
		boolean r = mainFrame.showEditorAsDialog(strand);
		if (!r) {
			strandRefresh(strand);
			setModified();
		}
	}

	/**
	 * set the deploy button
	 */
	private void setHiden(boolean b) {
		//LOG.trace(TT + ".setHiden(b=" + (b ? "true" : "false") + ") hiden=" + (hiden ? "true" : "false"));
		hiden = b;
		tb2.setVisible(hiden);
		btHide.setIcon(IconUtil.getIconSmall((hiden ? ICONS.K.DP_UP : ICONS.K.DP_DOWN)));
		App.preferences.setBoolean(Pref.KEY.TYPIST_SHOWBAR, hiden);
	}

	private void reviewInit() {
		//LOG.trace(TT + ".reviewInit()");
		htTexte.wysEditorGet().getToolBar2().addSeparator();
		reviewPanel = new ReviewPanel(mainFrame, this);
		reviewPanel.setVisible(book.getReview());
	}

	/**
	 * initialize the endnote functions
	 *
	 * @return a JButton for activate the liste of Endnotes
	 */
	@SuppressWarnings("unchecked")
	private void initEndnote() {
		//LOG.trace(TT + ".initEndnote(textPanel)");
		endnotes = Endnote.find(mainFrame, 0, scene);
		JButton bten = Endnote.getButtonAdd(e -> {
			Endnote.createEndnote(mainFrame, 0, scene, htTexte);
			endnotes = Endnote.find(mainFrame, 0, scene);
			btEndnotes.setEnabled(!endnotes.isEmpty());
		});
		htTexte.wysEditorGet().getToolBar2().add(bten);
		btEndnotes = Endnote.getButtonList(e -> ListEndnotesDlg.showDlg(mainFrame, 0, scene));
		btEndnotes.setEnabled(!endnotes.isEmpty());
	}

	private void changeCkReview() {
		book.setReview(!book.getReview());
		book.save();
		reviewPanel.setVisible(book.getReview());
	}

}
