/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.typist;

import i18n.I18N;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import storybook.App;
import storybook.Pref;
import storybook.edit.EditScenario;
import storybook.edit.Editor;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.SceneDAO;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Scene;
import storybook.tools.Markdown;
import storybook.tools.html.Html;
import storybook.tools.net.Net;
import storybook.tools.swing.SwingUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.combobox.ComboModelSceneState;
import storybook.ui.combobox.ComboUtil;
import storybook.ui.dialog.EntitiesListDlg;
import storybook.ui.interfaces.IRefreshable;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.renderer.lcr.LCRSceneState;

/**
 *
 * @author favdb
 */
public class TypistScenario extends AbstractPanel implements ActionListener, IRefreshable {

	private Scene scene;
	private boolean modified;
	private int nbMod = 0;
	private JButton btNext;
	private JButton btPrevious;
	private JButton btScreen;
	private JButton btSave;
	private JButton btIgnore;
	private JPanel left;
	private JPanel pPersons;
	private JTextArea taDescription;
	private JTextArea taNotes;
	private JComboBox cbStage;
	private JComboBox cbScenes;
	private JComboBox cbStatus;
	private JComboBox cbMoment;
	private JComboBox cbStart;
	private JComboBox cbEnd;
	private JComboBox cbLocation;
	private JTextField tfName = new JTextField();
	private JTextField tfPitch;
	private JRadioButton rbLocInt;
	private Markdown mdEdit = new Markdown("ScenarioPanel");
	private JButton btFirst;
	private JButton btLast;
	private EditScenario scenario;

	public enum CMD {
		BT_EXIT, // exit oStorybook
		BT_HELP, // help about Markdown syntax
		BT_IDEA, // add new idea
		BT_PERSON_NEW, // add new person and add a button
		BT_IGNORE_CHANGE,// ignore changes and restore original current scene
		BT_SCENE_FIRST, // goto the first scene
		BT_SCENE_LAST, // goto the last scene
		BT_SCENE_PREVIOUS, //goto previous scene
		BT_SCENE_NEXT, //goto next scene
		BT_SCENE_NEW,// add new scene
		BT_SCENE_SAVE,// save changes
		BT_SCREEN, // change full/normal screen
		BT_STAGE,// call browser to online help about assistant.stage
		CB_SCENES,// list of scenes
		CB_STAGE,// list of stage
		CB_STATUS,// list of status
		// scenario
		BT_ADD_LOCATION,
		CB_MOMENT,
		CB_LOC,
		CB_IN,
		CB_OUT,
		CB_LOCATIONS,
		RB_INT,
		// communs
		PRINT,// printing current scene
		REFRESH,// refresh current scene
		NONE;

		@Override
		public String toString() {
			return (name().toLowerCase());
		}
	}

	public static CMD getCMD(String str) {
		for (CMD a : CMD.values()) {
			if (a.toString().equals(str)) {
				return (a);
			}
		}
		return (CMD.NONE);
	}

	public TypistScenario(MainFrame mainFrame) {
		super(mainFrame);
		this.scene = (Scene) EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE).get(0);
		initAll();
	}

	public TypistScenario(MainFrame mainFrame, Scene scene) {
		super(mainFrame);
		this.scene = scene;
		initAll();
	}

	@Override
	public void init() {
		this.withPart = false;
		App.preferences.setBoolean(Pref.KEY.TYPIST_USE, true);
	}

	@Override
	public void initUi() {
		//LOG.trace(TT+".initUi()");
		if (this.getComponentCount() > 0) {
			this.removeAll();
		}
		this.setLayout(new MigLayout("wrap, hidemode 3, top, ins 1 1 1 1", "[grow][65%][20%]"));
		//no part to select
		initToolbar();
		//combobox scene
		cbScenes = Ui.initComboBox(CMD.CB_SCENES.toString(), "",
				EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE), scene, !EMPTY, !ALL, this);
		toolbar.add(cbScenes);
		toolbar.add(Ui.initButton(CMD.BT_SCENE_NEW.toString(), "", ICONS.K.NEW, "scene.new", this));
		//navigation dans les scènes
		//first scene
		btFirst = Ui.initButton(CMD.BT_SCENE_FIRST.toString(), "", ICONS.K.NAV_FIRST, "export.nav.first", this);
		btFirst.setEnabled(!isSceneFirst());
		toolbar.add(btFirst);
		//previous scene
		btPrevious = Ui.initButton(CMD.BT_SCENE_PREVIOUS.toString(), "", ICONS.K.NAV_PREV, "export.nav.previous", this);
		btPrevious.setEnabled(false);
		toolbar.add(btPrevious);
		btNext = Ui.initButton(CMD.BT_SCENE_NEXT.toString(), "", ICONS.K.NAV_NEXT, "export.nav.next", this);
		int n = Book.getNbScenes(mainFrame);
		if (n < 1) {
			btNext.setEnabled(false);
		}
		toolbar.add(btNext);
		//dernière scène
		btLast = Ui.initButton(CMD.BT_SCENE_LAST.toString(), "", ICONS.K.NAV_LAST, "export.nav.last", this);
		btLast.setEnabled(!isSceneLast());
		toolbar.add(btLast);
		toolbar.add(new JToolBar.Separator());
		//enregistrer les modifications
		btSave = Ui.initButton(CMD.BT_SCENE_SAVE.toString(), "", ICONS.K.F_SAVE, "save.changes", this);
		toolbar.add(btSave);
		//ignorer les modifications
		btIgnore = Ui.initButton(CMD.BT_IGNORE_CHANGE.toString(), "", ICONS.K.REFRESH, "discard.changes", this);
		toolbar.add(btIgnore);
		JButton btHelp = Ui.initButton(CMD.BT_HELP.toString(), "", ICONS.K.HELP, "markdown", this);
		toolbar.add(btHelp);
		toolbar.add(new JLabel(" "), MIG.get(MIG.PUSH, MIG.GROWX));
		add(toolbar, MIG.get(MIG.SPAN, MIG.SPLIT2, MIG.GROWX));
		JToolBar tbRight = new JToolBar();
		tbRight.setLayout(new MigLayout());
		tbRight.setBorder(BorderFactory.createEmptyBorder());
		//tbRight.setOpaque(false);
		tbRight.setFloatable(false);
		//sortie du mode typist
		btScreen = Ui.initButton(CMD.BT_SCREEN.toString(), "", ICONS.K.SCREEN_NORMAL, "screen.normal", this);
		tbRight.add(btScreen);
		// idea
		JButton ideaButton = Ui.initButton(CMD.BT_IDEA.toString(), "", ICONS.K.ENT_IDEA, "foi.new", this);
		tbRight.add(ideaButton);
		// quit
		JButton exitButton = Ui.initButton(CMD.BT_EXIT.toString(), "", ICONS.K.EXIT, "exit", this);
		tbRight.add(exitButton);
		add(tbRight, MIG.get(MIG.SPAN, MIG.RIGHT, MIG.WRAP));
		add(initHeader(), MIG.get(MIG.SPAN, MIG.GROW, MIG.WRAP));
		add(initLeft(), MIG.get(MIG.GROW, MIG.TOP));
		add(initMiddle());
		add(initRight(), MIG.GROWY);
		add(initFooter(), MIG.SPAN);
		if (scene == null) {
			sceneSet(0);
		} else {
			sceneSet(scene);
		}
		mdEdit.initSpelling(mainFrame);
	}

	@SuppressWarnings("unchecked")
	private JPanel initHeader() {
		JPanel head = new JPanel(new MigLayout(MIG.get(MIG.FILL, MIG.HIDEMODE3)));
		head.setBorder(BorderFactory.createRaisedBevelBorder());
		tfName = initTextField("name", 16, "");
		tfName.setEditable(false);
		head.add(new JLabel(I18N.getColonMsg("name")), MIG.get(MIG.SPAN, "split 4"));
		head.add(tfName);
		cbStatus = new JComboBox(new ComboModelSceneState());
		cbStatus.setName(CMD.CB_STATUS.toString());
		cbStatus.setSelectedIndex(6);
		cbStatus.setRenderer(new LCRSceneState());
		head.add(new JLabel(I18N.getColonMsg("status")), MIG.get(MIG.SPAN, MIG.SPLIT2));
		head.add(cbStatus, MIG.WRAP);
		cbStage = new JComboBox();
		cbStage.setName("cbStage");
		ComboUtil.loadCbStage(mainFrame, cbStage, "");
		head.add(new JLabel(I18N.getColonMsg("assistant.stage")), MIG.get(MIG.SPAN, "split 3"));
		head.add(cbStage, MIG.SPLIT2);
		head.add(Ui.initButton(CMD.BT_STAGE.toString(), "", ICONS.K.HELP, "assistant.stage.help", this));
		scenario = new EditScenario(this);
		head.add(scenario, MIG.SPAN);
		return (head);
	}

	private JPanel initLeft() {
		left = new JPanel(new MigLayout(""));
		left.setBorder(BorderFactory.createTitledBorder(I18N.getColonMsg("persons")));
		JButton bt = Ui.initButton(CMD.BT_PERSON_NEW.toString(), "", ICONS.K.NEW, "person.new", this);
		left.add(bt, "span, growx");
		pPersons = new JPanel(new MigLayout(MIG.get(MIG.INS0, MIG.WRAP)));
		JScrollPane personScroll = new JScrollPane(pPersons);
		left.add(personScroll, MIG.get(MIG.SPAN, MIG.GROW, MIG.WRAP));
		return (left);
	}

	private JPanel initMiddle() {
		JPanel middle = new JPanel(new MigLayout(MIG.HIDEMODE3));
		//the text editor
		mdEdit = new Markdown("ScenarioPanel", Html.TYPE, "");
		mdEdit.setCaretPosition(0);
		mdEdit.setCallback(this);
		middle.add(mdEdit, MIG.GROW);
		mdEdit.setView(Markdown.VIEW.TEXT);
		return (middle);
	}

	private JPanel initRight() {
		//LOG.trace(TT+".initRght()");
		JPanel right = new JPanel(new MigLayout(MIG.WRAP));
		taDescription = initTextArea(right, "description", "");
		taNotes = initTextArea(right, "notes", "");
		return (right);
	}

	private JPanel initFooter() {
		//LOG.trace(TT+".initFooter()");
		JPanel foot = new JPanel(new MigLayout());
		return (foot);
	}

	private boolean isSceneFirst() {
		return (scene == null);
	}

	private boolean isSceneLast() {
		return (scene == null);
	}

	private void locationsLoad() {
		EntityUtil.fillCB(mainFrame, cbLocation, Book.TYPE.LOCATION, null, false, false);
	}

	public Markdown getMdEdit() {
		return mdEdit;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		String propName = evt.getPropertyName();
		String types = "scene, location, person, plot, photo";
		if (propName.contains("_")) {
			String m[] = propName.split("_");
			if (!types.contains(m[0])) {
				return;
			}
			switch (Book.getTYPE(m[0])) {
				case SCENE:
					scenesLoad();
					if (m[1].equalsIgnoreCase("new")) {
						cbScenes.setSelectedItem((Scene) evt.getNewValue());
					}
					break;
				case PERSON:
					personsLoad();
					break;
				case LOCATION:
					locationsLoad();
					break;
				default:
					break;
			}
		}
	}

	public void sceneRefresh(Scene scene) {
		if (scene != null) {
			this.scene = scene;
			sceneRefresh();
			scenario.setScene(scene);
		}
	}

	public void sceneRefresh() {
		resetListener();
		if (scene == null) {
			tfName.setText("");
			mdEdit.setText("");
			cbStage.setSelectedIndex(-1);
			cbStatus.setSelectedIndex(-1);
		} else {
			tfName.setText(scene.getName());
			if (scene.getSummary() != null) {
				if (scene.getSummary().contains("<p")) {
					mdEdit.setText(Markdown.toMarkdown(scene.getSummary()));
				} else {
					mdEdit.setText(scene.getSummary());
				}
			} else {
				mdEdit.setText("");
			}
			cbStage.setSelectedIndex(scene.getScenario_stage());
			cbStatus.setSelectedIndex(scene.getStatus());
			scenario.setScene(scene);
			mainFrame.sceneSetLast(scene);
			mdEdit.setHeader(scene, book.info.scenarioGet());
		}
		mdEdit.setCaretPosition(0);
		personsLoad();
		left.revalidate();
		cbScenes.setSelectedItem(scene);
		enableNav(cbScenes.getSelectedIndex());
		resetModified();
		setListener();
	}

	private void setListener() {
		cbScenes.addActionListener(this);
		cbStage.addActionListener(this);
		cbStatus.addActionListener(this);
	}

	private void resetListener() {
		cbScenes.removeActionListener(this);
		cbStage.removeActionListener(this);
		cbStatus.removeActionListener(this);
	}

	public JTextField initTextField(String title, int len, String val) {
		JTextField tf = new JTextField();
		tf.setName(title);
		tf.setText(val);
		if (len > 0) {
			tf.setColumns(len);
		}
		return (tf);
	}

	public JTextArea initTextArea(JPanel p, String title, String val) {
		p.add(new JLabel(I18N.getColonMsg(title)));
		JTextArea ta = new JTextArea();
		ta.setName(title);
		if (val != null) {
			ta.setText(val);
		}
		ta.setLineWrap(true);
		ta.setWrapStyleWord(true);
		JScrollPane scroller = new JScrollPane(ta);
		SwingUtil.setMaxPreferredSize(scroller);
		p.add(scroller);
		return (ta);
	}

	@Override
	@SuppressWarnings({"unchecked", "unchecked"})
	public void actionPerformed(ActionEvent evt) {
		//LOG.trace(TT+".actionPerformed(evt=" + evt.toString() + ")");
		if (evt.getSource() instanceof JButton) {
			JButton btn = (JButton) evt.getSource();
			int n;
			switch (getCMD(btn.getName())) {
				case BT_HELP:
					JOptionPane.showMessageDialog(this, I18N.getMsg("markdown.help"));
					break;
				case BT_EXIT:
					if (modified) {
						getScenarioData();
						save();
						mainFrame.saveFile(true);
					}
					SwingUtilities.invokeLater(() -> {
						mainFrame.close(true);
					});
					break;
				case BT_PERSON_NEW:
					personNew();
					break;
				case BT_SCENE_FIRST:
					if (modified) {
						getScenarioData();
						save();
					}
					getSceneFirst();
					sceneRefresh();
					break;
				case BT_SCENE_LAST:
					if (modified) {
						scenario.getScenarioData(scene);
						save();
					}
					getSceneLast();
					sceneRefresh();
					break;
				case BT_SCENE_PREVIOUS:
					if (modified) {
						scenario.getScenarioData(scene);
						save();
					}
					getScenePrior();
					sceneRefresh();
					break;
				case BT_SCENE_NEXT:
					if (modified) {
						scenario.getScenarioData(scene);
						save();
					}
					getSceneNext();
					sceneRefresh();
					break;
				case BT_SCENE_NEW:
					if (scene != null && checkModified() == JOptionPane.CANCEL_OPTION) {
						return;
					}
					sceneNew();
					break;
				case BT_IGNORE_CHANGE:
					modified = false;
					scene = (Scene) EntityUtil.getSceneById(mainFrame, scene.getId());
					sceneRefresh();
					break;
				case BT_SCENE_SAVE:
					scene = getScenarioData();
					save();
					break;
				case BT_SCREEN:
					if (modified) {
						getScenarioData();
						save();
					}
					close();
					break;
				case BT_IDEA:
					mainFrame.showEditorAsDialog(new Idea());
					break;
				case BT_STAGE:
					Net.openBrowser(I18N.getMsg("assistant.stage.web"));
					break;
				case BT_ADD_LOCATION:
					Location entity = new Location();
					JDialog dlg = new JDialog(mainFrame.fullFrame, true);
					Editor editor = new Editor(mainFrame, entity, dlg);
					dlg.setTitle(I18N.getMsg("editor"));
					dlg.add(editor);
					Dimension pos = SwingUtil.getDlgPosition(entity);
					Dimension size = SwingUtil.getDlgSize(entity);
					dlg.setSize((size != null ? size.height : this.getWidth() / 2),
							(size != null ? size.width : 680));
					if (pos != null) {
						dlg.setLocation(pos.height, pos.width);
					} else {
						dlg.setLocationRelativeTo(this);
					}
					dlg.setVisible(true);
					SwingUtil.saveDlgPosition(dlg, entity);
					if (!editor.canceled) {
						locationsLoad();
						cbLocation.setSelectedItem(cbLocation.getItemCount() - 1);
					}
					break;
				default:
					break;
			}
		} else if (evt.getSource() instanceof JComboBox) {
			JComboBox cb = (JComboBox) evt.getSource();
			switch (getCMD(cb.getName())) {
				case CB_SCENES:
					checkModified();
					if (cbScenes.getSelectedIndex() != -1) {
						scene = (Scene) cbScenes.getSelectedItem();
					}
					sceneRefresh();
					enableNav(cbScenes.getSelectedIndex());
					break;
				case CB_LOCATIONS:
				case CB_LOC:
				case CB_MOMENT:
				case CB_IN:
				case CB_OUT:
					getScenarioData();
					mdEdit.setHeader(scene, book.info.scenarioGet());
					modified = true;
					break;
				default:
					break;
			}
		} else if (evt.getSource() instanceof JRadioButton) {
			getScenarioData();
			mdEdit.setHeader(scene, book.info.scenarioGet());
			modified = true;
		}
		if (modified) {
			this.btSave.setEnabled(true);
			this.btIgnore.setEnabled(true);
		}
	}

	public int askModified() {
		if (scene != null && mdEdit.isModified()) {
			if (mdEdit.getText().length() > 32767) {
				String str = I18N.getMsg("editor.text_too_large");
				JOptionPane.showMessageDialog(
						this,
						str,
						I18N.getMsg("editor"),
						JOptionPane.ERROR_MESSAGE);
				return (JOptionPane.CANCEL_OPTION);
			}
			modified = true;
			scene.setSummary(mdEdit.getText());
			mdEdit.resetModified();
		}
		if (modified) {
			getScenarioData();
			save();
		}
		return (JOptionPane.YES_OPTION);
	}

	public Scene getScene() {
		return this.scene;
	}

	public void closeScene() {
		// empty
	}

	private void close() {
		mainFrame.typistActivate();
	}

	private void enableNav(int n) {
		btFirst.setEnabled(n > 0);
		btPrevious.setEnabled(n > 0);
		btNext.setEnabled(n < cbScenes.getItemCount() - 1);
		btLast.setEnabled(n < cbScenes.getItemCount() - 1);
	}

	private void personNew() {
		EntitiesListDlg dlg = new EntitiesListDlg(mainFrame, scene, "person");
		dlg.setModal(true);
		dlg.setVisible(true);
		if (!dlg.isCanceled()) {
			scene.setPersons(dlg.getPersons());
			personsLoad();
			setModified();
		}
	}

	private JButton personInitButton(Person person) {
		JButton bt = new JButton(person.getIcon());
		bt.setName("btDialog_" + person.getId());
		bt.setText(person.getAbbr());
		bt.setToolTipText(Html.intoHTML(person.toDetail(false)));
		bt.addActionListener(evt -> addDialog(person));
		return (bt);
	}

	private void addDialog(AbstractEntity entity) {
		String text = mdEdit.getText();
		if (entity == null) {
			return;
		}
		if (scene == null) {
			return;
		}
		if (!text.isEmpty() && !text.endsWith("\n")) {
			text += "\n";
		}
		if (!text.isEmpty() && !text.endsWith("\n\n")) {
			text += "\n";
		}
		text += "[" + entity.getName().trim() + "] ";
		mdEdit.setText(text);
		mdEdit.setFocus();
		setModified();
	}

	private Scene getSceneNext() {
		int i = cbScenes.getSelectedIndex();
		if (i == cbScenes.getItemCount() - 1) {
			return (scene);
		}
		scene = (Scene) cbScenes.getItemAt(i + 1);
		return (scene);
	}

	private Scene getScenePrior() {
		int i = cbScenes.getSelectedIndex();
		if (i < 1) {
			return (scene);
		}
		scene = (Scene) cbScenes.getItemAt(i - 1);
		return (scene);
	}

	private Scene getSceneFirst() {
		scene = (Scene) cbScenes.getItemAt(0);
		return (scene);
	}

	private Scene getSceneLast() {
		int i = cbScenes.getItemCount();
		if (i < 1) {
			return (scene);
		}
		scene = (Scene) cbScenes.getItemAt(i - 1);
		return (scene);
	}

	public Scene getScenarioData() {
		scene.setStatus(cbStatus.getSelectedIndex());
		scene.setScenario_stage(cbStage.getSelectedIndex());
		List<Location> locs = new ArrayList<>();
		if (cbLocation.getSelectedItem() != null) {
			locs.add((Location) cbLocation.getSelectedItem());
		}
		scene.setLocations(locs);
		scene.setScenario_pitch(tfPitch.getText());
		scene.setScenario_moment(cbMoment.getSelectedIndex());
		scene.setScenario_loc((rbLocInt.isSelected() ? 1 : 2));
		scene.setScenario_start(cbStart.getSelectedIndex());
		scene.setScenario_end(cbEnd.getSelectedIndex());
		scene.setSummary(mdEdit.getText());
		scene.setDescription(taDescription.getText());
		scene.setNotes(taNotes.getText());
		return (scene);
	}

	public int checkModified() {
		if (scene == null) {
			return (JOptionPane.CANCEL_OPTION);
		}
		if (mdEdit.getText().length() > 32767) {
			String str = I18N.getMsg("editor.text_too_large");
			JOptionPane.showMessageDialog(
					this,
					str,
					I18N.getMsg("editor"),
					JOptionPane.ERROR_MESSAGE);
			return (JOptionPane.CANCEL_OPTION);
		}
		if (modified) {
			scenario.getScenarioData(scene);
			save();
		}
		return (JOptionPane.YES_OPTION);
	}

	private void save() {
		resetModified();
		if (scene.getId() < 0) {
			mainFrame.getBookController().newEntity(scene);
		} else {
			mainFrame.getBookController().updateEntity(scene);
		}
	}

	@SuppressWarnings("unchecked")
	private void sceneNew() {
		Scene s = new Scene();
		if (scene != null) {
			s.setChapter(scene.getChapter());
			s.setStrand(scene.getStrand());
		}
		s.setSceneno(Book.getNbScenes(mainFrame) + 1);
		String title = I18N.getMsg("scene") + " " + s.getSceneno();
		s.setName(title);
		s.setTitle(title);
		mainFrame.getBookModel().SCENE_New(s);
		scenesLoad();
		cbScenes.setSelectedIndex(cbScenes.getItemCount() - 1);
		sceneSet(cbScenes.getSelectedIndex());
	}

	private void sceneSet(Scene s) {
		cbScenes.setSelectedItem(s);
		if (cbScenes.getSelectedIndex() < 0) {
			return;
		}
		sceneSet(cbScenes.getSelectedIndex());
	}

	private void sceneSet(int n) {
		if (n < 0) {
			return;
		}
		if (modified) {
			scenario.getScenarioData(scene);
			//getScenarioData();
			save();
		}
		Scene s = (Scene) cbScenes.getItemAt(n);
		if (s != null) {
			cbScenes.setSelectedItem(s);
			sceneRefresh(s);
			mdEdit.setCaretPosition(0);
		}
		enableNav(n);
	}

	private void personsLoad() {
		pPersons.removeAll();
		if (scene != null && !scene.getPersons().isEmpty()) {
			scene.getPersons().forEach(person -> {
				if (person != null) {
					pPersons.add(personInitButton(person), MIG.get(MIG.CENTER, MIG.GROWX));
				}
			});
		}
	}

	@SuppressWarnings("unchecked")
	private void scenesLoad() {
		cbScenes.removeActionListener(this);
		DefaultComboBoxModel combo = (DefaultComboBoxModel) cbScenes.getModel();
		combo.removeAllElements();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findAll();
		for (Scene entity : scenes) {
			combo.addElement(entity);
		}
		cbScenes.addActionListener(this);
	}

	public void setModified() {
		modified = true;
		enableSave();
		nbMod++;
		if (nbMod > 10) {
			save();
			nbMod = 0;
		}
	}

	public void resetModified() {
		modified = false;
		enableSave();
	}

	private void enableSave() {
		btSave.setEnabled(modified);
		btIgnore.setEnabled(modified);
	}

	private JPanel initScenario() {
		//LOG.trace(TT+".scenarioInit()");
		JPanel p = new JPanel(new MigLayout(MIG.HIDEMODE3));
		p.setBorder(BorderFactory.createTitledBorder(I18N.getMsg("scenario")));
		// pitch
		p.add(new JLabel(I18N.getColonMsg("scenario.pitch")), MIG.get(MIG.SPAN, MIG.SPLIT2));
		tfPitch = new JTextField();
		tfPitch.setName("tfPitch");
		p.add(tfPitch, MIG.GROWX);
		// liste des moments
		p.add(new JLabel(I18N.getColonMsg("scenario.moment")));
		cbMoment = initCbScenario(CMD.CB_MOMENT);
		p.add(cbMoment);
		// radio des INT/EXT
		p.add(new JLabel(I18N.getColonMsg("location")));
		rbLocInt = new JRadioButton(I18N.getMsg("scenario.loc.int"));
		rbLocInt.setName("rbInt");
		JRadioButton rbLocExt = new JRadioButton(I18N.getMsg("scenario.loc.ext"));
		rbLocExt.setName("rbExt");
		ButtonGroup group = new ButtonGroup();
		group.add(rbLocInt);
		group.add(rbLocExt);
		p.add(rbLocInt);
		p.add(rbLocExt);
		// liste des lieux
		cbLocation = new JComboBox();
		cbLocation.setName(CMD.CB_LOCATIONS.toString());
		locationsLoad();
		p.add(cbLocation);
		JButton btLoc = Ui.initButton(CMD.BT_ADD_LOCATION.toString(), "", ICONS.K.NEW, "location.add", this);
		p.add(btLoc, "wrap");
		// liste de la transition du début
		p.add(new JLabel(I18N.getColonMsg("scenario.start")));
		cbStart = initCbScenario(CMD.CB_IN);
		p.add(cbStart);
		// liste de la transition de fin
		p.add(new JLabel(I18N.getColonMsg("scenario.end")), MIG.get(MIG.SPAN, MIG.SPLIT2, MIG.RIGHT));
		cbEnd = initCbScenario(CMD.CB_OUT);
		p.add(cbEnd);
		return (p);
	}

	private JComboBox initCbLocation() {
		return (cbLocation);
	}

	@SuppressWarnings("unchecked")
	public JComboBox initCbScenario(CMD action) {
		//LOG.trace(TT+".initCbScenario("+action.toString()+")");
		String type = action.toString().toLowerCase().replaceFirst("cb_", "");
		JComboBox cb = new JComboBox();
		cb.setName(action.toString());
		for (int i = 0;; i++) {
			String s = findScenarioKey(type, i);
			if (s.isEmpty()) {
				break;
			}
			cb.addItem(s);
		}
		return (cb);
	}

	public String findScenarioKey(String type, int i) {
		//LOG.trace(TT+".findKey(type=" + type + ", i=" + i + ")");
		String key = "scenario." + type + "." + String.format("%02d", i);
		try {
			String msg = I18N.getMsg(key);
			if (msg.equals("!" + key + "!")) {
				return ("");
			}
			return (msg);
		} catch (Exception ex) {
			return ("");
		}
	}

}
