/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.table;

import assistant.Assistant;
import i18n.I18N;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.DefaultListSelectionModel;
import javax.swing.DefaultRowSorter;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.RowSorter;
import javax.swing.RowSorter.SortKey;
import javax.swing.ScrollPaneConstants;
import javax.swing.SortOrder;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import net.infonode.docking.View;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.ICONS.K;
import resources.icons.IconUtil;
import storybook.App;
import storybook.action.DeleteEntityAction;
import storybook.ctrl.ActKey;
import storybook.ctrl.Ctrl;
import static storybook.ctrl.Ctrl.PROPS.UPDATE;
import storybook.exim.EXIM;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.book.BookUtil;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.DB_DATA;
import storybook.model.hbn.entity.DB_DATA.DATA;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Internal;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Scene;
import storybook.tools.LOG;
import storybook.tools.ListUtil;
import storybook.tools.TextUtil;
import storybook.tools.file.XEditorFile;
import storybook.tools.html.Html;
import static storybook.tools.swing.SwingUtil.showModalDialog;
import storybook.tools.swing.js.JSColumnControlIcon;
import storybook.tools.swing.js.JSTable;
import storybook.tools.swing.js.JSToolBar;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.SbView;
import storybook.ui.SbView.VIEWNAME;
import storybook.ui.Ui;
import storybook.ui.dialog.ConfirmDeleteDlg;
import storybook.ui.dialog.ScenesChange;
import storybook.ui.panel.AbstractPanel;
import static storybook.ui.table.AbsColumn.*;

/**
 * abstract tabe class contains an optional header, a JTable, an optional footer
 *
 * @author favdb
 */
@SuppressWarnings("serial")
public abstract class AbstractTable extends AbstractPanel
		implements ActionListener, ListSelectionListener, TableModelListener {

	public static final String TT = "AbstractTable",
			TABLE_HEADER = "Table",
			BT_EDIT = "BtEdit",
			BT_NEW = "BtNew",
			BT_COPY = "BtCopy",
			BT_DELETE = "BtDelete";
	private final Book.TYPE type;
	protected List<AbsColumn> columns;
	private Part currentPart;
	protected JSTable table;
	protected DefaultTableModel tableModel;
	protected boolean allowMultiDelete = true;
	public JButton btNew,
			btDelete,
			btEdit,
			btCopy,
			btExternal;
	private int currentRow;
	private JButton btCtrlColumn;
	public ObjectifPanel pObjectif;
	public JComboBox cbCategories;
	private AbstractEntity currentEntity;

	/**
	 * class definition
	 *
	 * @param mainFrame
	 * @param type
	 */
	protected AbstractTable(MainFrame mainFrame, Book.TYPE type) {
		super(mainFrame);
		this.type = type;
		this.setName(type.toString());
		ctrl = mainFrame.getBookController();
		currentPart = null;
		initColumns();
	}

	/**
	 * get the entity type
	 *
	 * @return
	 */
	public Book.TYPE getType() {
		return type;
	}

	/**
	 * initialize Columns
	 *
	 */
	private void initColumns() {
		AbstractEntity entity = BookUtil.getNewEntity(mainFrame, type);
		columns = getColumns(entity);
	}

	/**
	 * get entity columnns
	 *
	 * @param e
	 * @return
	 */
	public List<AbsColumn> getColumns(AbstractEntity e) {
		List<AbsColumn> cols = new ArrayList<>();
		cols.add(new AbsColumn(mainFrame, cols, DATA.ID, NUM_LONG, TCR_HIDE, AL_CENTER));
		if (!(e instanceof Endnote)) {
			cols.add(new AbsColumn(mainFrame, cols, DATA.NAME));
		}
		return (cols);
	}

	/**
	 * get common entity columns
	 *
	 * @param columns
	 * @param entity
	 */
	public void getColumnsEnd(List<AbsColumn> columns, AbstractEntity entity) {
		String common = "000";
		if (entity != null) {
			if (entity.getCommon() != null) {
				common = entity.getCommon();
			}
			if (common.length() < 3) {
				common = common + "000";
			}
		}
		if (common.charAt(0) == '1') {
			columns.add(new AbsColumn(mainFrame, columns, DB_DATA.DATA.DESCRIPTION));
		}
		if (common.charAt(1) == '1') {
			columns.add(new AbsColumn(mainFrame, columns, DB_DATA.DATA.NOTES));
		}
		if (common.charAt(2) == '1') {
			columns.add(new AbsColumn(mainFrame, columns, DB_DATA.DATA.ASSISTANT));
		}
	}

	/**
	 * get a new empty entity
	 *
	 * @return
	 */
	protected AbstractEntity getNewEntity() {
		//LOG.trace(TT + "getNewEntity()");
		return (BookUtil.getNewEntity(mainFrame, type));
	}

	/**
	 * get the entity with an ID
	 *
	 * @param id
	 * @return
	 */
	protected abstract AbstractEntity getEntity(Long id);

	/**
	 * send an entity to edit from a row
	 *
	 * @param row
	 */
	protected void sendSetEntityToEdit(int row) {
		if (row == -1) {
			return;
		}
		currentRow = row;
		AbstractEntity e = getEntityFromRow(row);
		if (e == null || e instanceof Internal) {
			return;
		}
		boolean r = mainFrame.showEditorAsDialog(e);
		if (!r) {
			updateEntity(e);
		}
	}

	/**
	 * send an entity to edit from entity
	 *
	 * @param entity
	 */
	protected void sendSetNewEntityToEdit(AbstractEntity entity) {
		//LOG.trace(TT + ".sendSetNewEntityToEdit(entity)");
		if (entity instanceof Internal) {
			return;
		}
		boolean r = mainFrame.showEditorAsDialog(entity);
		if (!r) {
			newEntity(entity);
		}
	}

	/**
	 * send an entity to delete from a row
	 *
	 * @param row
	 */
	protected synchronized void sendDeleteEntity(int row) {
		AbstractEntity e = getEntityFromRow(row);
		deleteEntity(e);
		if (currentRow > table.getRowCount() - 1) {
			currentRow = 0;
		}
	}

	/**
	 * send entities to delete from multiple rows
	 *
	 * @param entities
	 */
	protected synchronized void sendDeleteEntities(List<AbstractEntity> entities) {
		for (AbstractEntity e : entities) {
			mainFrame.getBookModel().ENTITY_Delete(e);
		}
		mainFrame.getBookModel().fireAgain();
	}

	protected synchronized void copyTo(int r) {
		// empty, only used for Chapter and Scene tables
	}

	/**
	 * property local change
	 *
	 * @param evt
	 */
	protected abstract void modelPropertyChangeLocal(PropertyChangeEvent evt);

	/**
	 * action to sort up this entity row
	 *
	 * @param row
	 */
	protected void sendOrderUpEntity(int row) {
	}

	/**
	 * action to sort down this entity row
	 *
	 * @param row
	 */
	protected void sendOrderDownEntity(int row) {
	}

	/**
	 * initialize the toolbar
	 *
	 * @return
	 */
	@Override
	public JToolBar initToolbar() {
		super.initToolbar();
		return toolbar;
	}

	/**
	 * get the current Part
	 *
	 * @return
	 */
	public Part getCurrentPart() {
		if (cbPartFilter == null) {
			currentPart = null;
		} else if (cbPartFilter.getSelectedIndex() == 0) {
			currentPart = null;
		} else {
			currentPart = (Part) cbPartFilter.getSelectedItem();
		}
		return (currentPart);
	}

	/**
	 * get all entities
	 *
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<AbstractEntity> getAllEntities() {
		//LOG.trace(TT+".getAllEntities() "
		//		+"type="+type.name()
		//		+", currentPart="+(currentPart==null?"null":currentPart.toString()));
		List list = EntityUtil.findEntities(mainFrame, type);
		return (list);
	}

	/**
	 * Property chage action
	 *
	 * @param evt
	 */
	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		String propName = evt.getPropertyName();
		//LOG.trace(TT + ".modelPropertyChange(evt=" + evt.toString() + ")");
		ActKey act = new ActKey(evt);
		if ("UPDATE".equals(act.getCmd())) {
			modelPropertyChangeLocal(evt);
			return;
		}
		switch (Ctrl.getPROPS(propName)) {
			case REFRESH:
				View newView = (View) evt.getNewValue();
				View view = (View) getParent().getParent();
				if (view == newView) {
					refreshAll();
				}
				return;
			case EXPORT:
				EXIM.exporter(mainFrame, (SbView) evt.getNewValue());
				return;
			case SHOWINFO:
				return;
			default:
		}
		if (getTableName().toLowerCase().contains(act.type.toLowerCase())) {
			switch (Ctrl.getPROPS(act.getCmd())) {
				case INIT:
					initTableModel(evt);
					break;
				case NEW:
					newEntity(evt);
					break;
				case DELETE:
					deleteEntity(evt);
					break;
				case UPDATE:
					updateEntity(evt);
					break;
				case ORDERUP:
					orderUpEntity(evt);
					break;
				case ORDERDOWN:
					orderDownEntity(evt);
					break;
				default:
					break;
			}
			modelPropertyChangeLocal(evt);
		}
		revalidate();
	}

	/**
	 * initialize the user interface
	 *
	 */
	@Override
	public void initUi() {
		//LOG.trace(TT+".initUi() for table "+type.toString());
		removeAll();
		setLayout(new MigLayout(MIG.get(MIG.FILL, MIG.WRAP, MIG.HIDEMODE3)));
		initToolbar();
		if (toolbar != null && toolbar.getComponentCount() > 0) {
			add(toolbar, MIG.GROWX);
		}
		initTable();
		initEditor();
		initRenderer();
		JScrollPane scroller = new JScrollPane(table);
		btCtrlColumn = new JButton(new JSColumnControlIcon());
		btCtrlColumn.addActionListener(e -> {
			table.changeVisibility(btCtrlColumn.getLocationOnScreen());
		});
		scroller.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER, btCtrlColumn);
		scroller.setPreferredSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
		//scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		add(scroller, MIG.GROW);
		if (type != Book.TYPE.INTERNAL) {
			add(initFooter(), MIG.get(MIG.SPLIT2, MIG.GROWX));
		}
		if (type == Book.TYPE.CHAPTER) {
			pObjectif = new ObjectifPanel(this);
			add(pObjectif, MIG.get(MIG.RIGHT));
		}
		table.getModel().addTableModelListener(this);
	}

	;
	/**
	 * initialize the table object
	 *
	 */
	private void initTable() {
		List<String> colNames = getColumnNames();
		tableModel = new DefaultTableModel(colNames.toArray(), 0);
		table = new JSTable();
		table.setFont(App.getInstance().fontGetDefault());
		table.setModel(tableModel);
		table.setName(this.getClass().getSimpleName());
		table.addMouseListener(new MyMouseAdapter());
		table.getSelectionModel().addListSelectionListener(this);
		// *** hot keys VK_ENTER to edit the row Entity
		table.getInputMap().put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
				"Enter");
		table.getInputMap().put(
				KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
				"CopyTo");
		AbstractAction actEnter = new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				int r = table.getSelectedRow();
				if (r != -1) {
					sendSetEntityToEdit(r);
				}
			}
		};
		table.getActionMap().put("Enter", actEnter);
		if (type.equals(Book.TYPE.CHAPTER) || type.equals(Book.TYPE.SCENE)) {
			AbstractAction actCopyTo = new AbstractAction() {
				@Override
				public void actionPerformed(ActionEvent ae) {
					int r = table.getSelectedRow();
					if (r != -1) {
						copyTo(r);
					}
				}
			};
			table.getActionMap().put("CopyTo", actCopyTo);
		}
		if (type != Book.TYPE.ENDNOTE) {
			table.registerKeyboardAction(this, BT_NEW,
					KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0, false),
					JComponent.WHEN_FOCUSED);
			table.registerKeyboardAction(this, BT_COPY,
					KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK, false),
					JComponent.WHEN_FOCUSED);
			table.registerKeyboardAction(this, BT_DELETE,
					KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0, false),
					JComponent.WHEN_FOCUSED);
		}
	}

	/**
	 * initialize the cells editors
	 *
	 */
	private void initEditor() {
		//LOG.trace(TT + ".initEditor()");
		for (AbsColumn sbcol : columns) {
			TableColumn tcol;
			try {
				tcol = table.getColumn(sbcol.toString());
				if (sbcol.getTCE() != null) {
					tcol.setCellEditor(sbcol.getTCE());
					table.setColumnEditable(sbcol.toString());
				}
			} catch (IllegalArgumentException ex) {
				LOG.err(TT + ".initEditor() Error tcol", ex);
			}
		}
	}

	/**
	 * initialize the cells renderers and comparators
	 *
	 */
	private void initRenderer() {
		TableRowSorter<TableModel> sorter = new TableRowSorter<>(table.getModel());
		for (AbsColumn sbcol : columns) {
			TableColumn tcol;
			try {
				tcol = table.getColumn(sbcol.toString());
			} catch (IllegalArgumentException ex) {
				LOG.err("AbstactTable.initUi() Error tcol");
				continue;
			}
			initCellRenderer(sbcol, tcol);
			if (sbcol.hasComparator()) {
				sorter.setComparator(tcol.getModelIndex(), sbcol.getComparator());
			}
			if (!sbcol.isSortable()) {
				sorter.setSortable(tcol.getModelIndex(), false);
			}
			// visible on start
			tcol.setMaxWidth(800);
			table.setColumnVisible(sbcol.toString(), !sbcol.isHideOnStart());
		}
		table.setRowSorter(sorter);
	}

	/**
	 * initialize a cell renderer
	 *
	 * @param col
	 * @param tcol
	 */
	private void initCellRenderer(AbsColumn col, TableColumn tcol) {
		DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
		if (col.getTCR() != null) {
			renderer = (DefaultTableCellRenderer) col.getTCR();
		}
		if (col.getAlign() == SwingConstants.CENTER) {
			renderer.setHorizontalAlignment(SwingConstants.CENTER);
		}
		tcol.setCellRenderer(renderer);
	}

	/**
	 * initialize the footer
	 *
	 * @return
	 */
	public JToolBar initFooter() {
		JToolBar footer = new JSToolBar(true);
		btNew = Ui.initButton(BT_NEW, "", K.NEW, "new", this);
		btNew.registerKeyboardAction(this,
				KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0, true),
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		footer.add(btNew);
		btEdit = Ui.initButton(BT_EDIT, "", K.EDIT, "edit", this);
		btEdit.setEnabled(false);
		footer.add(btEdit);
		btCopy = Ui.initButton(BT_COPY, "", K.EDIT_COPY, "copy", this);
		btCopy.setEnabled(false);
		footer.add(btCopy);
		btDelete = Ui.initButton(BT_DELETE, "", K.DELETE, "delete", this);
		btDelete.registerKeyboardAction(this,
				KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0, true),
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		btDelete.setEnabled(false);
		footer.add(btDelete);
		return footer;
	}

	/**
	 * remove all data from the table
	 *
	 */
	public void clearTable() {
		// clear table
		for (int i = tableModel.getRowCount() - 1; i >= 0; i--) {
			tableModel.removeRow(i);
		}
	}

	/**
	 * fill data in the table
	 *
	 */
	public void fillTable() {
		//LOG.trace(TT+".fillTable() for table="+this.getName());
		if (table == null) {
			return;
		}
		table.getModel().removeTableModelListener(this);
		currentRow = table.getSelectionModel().getMinSelectionIndex();
		DefaultTableModel dm = (DefaultTableModel) table.getModel();
		int rowCount = dm.getRowCount();
		for (int i = rowCount - 1; i >= 0; i--) {
			dm.removeRow(i);
		}
		List<AbstractEntity> ls = getAllEntities();
		// fill in data
		rowCount = 0;
		for (AbstractEntity e : ls) {
			List<Object> cols = getRow(e);
			tableModel.addRow(cols.toArray());
			rowCount++;
		}
		if (type == Book.TYPE.CHAPTER && pObjectif != null) {
			Integer nbTotalObjectif = 0, nbRealized = 0;
			for (AbstractEntity entity : ls) {
				nbTotalObjectif += ((Chapter) entity).getObjectiveChars();
				for (Scene s : EntityUtil.findScenes(mainFrame, (Chapter) entity)) {
					nbRealized += s.getChars();
				}
			}
			if (nbTotalObjectif > 0) {
				pObjectif.setTotal(nbTotalObjectif);
				pObjectif.setRealized(nbRealized);
				pObjectif.setVisible(true);
				this.revalidate();
			} else {
				pObjectif.setVisible(false);
			}
		}
		if (currentRow != -1) {
			table.getSelectionModel().setSelectionInterval(currentRow, currentRow);
		}
		table.getModel().addTableModelListener(this);
	}

	/**
	 * initialize table model
	 *
	 * @param evt
	 */
	protected void initTableModel(PropertyChangeEvent evt) {
		//LOG.trace(TT+".initTableModel("+evt.toString()+")");
		table.putClientProperty("MainFrame", mainFrame);
		fillTable();
	}

	/**
	 * get the table object
	 *
	 * @return the table
	 */
	public JTable getTable() {
		return table;
	}

	/**
	 * get the name of this table
	 *
	 * @return the table name
	 */
	public String getTableName() {
		return (this.getClass().getSimpleName());
	}

	/**
	 * get the column names
	 *
	 * @return the list of columns names
	 */
	public List<String> getColumnNames() {
		List<String> cols = new ArrayList<>();
		for (AbsColumn col : columns) {
			cols.add(col.toString());
		}
		return cols;
	}

	/**
	 * abstract for updating the row
	 *
	 * @param entity
	 */
	public abstract void updateRow(AbstractEntity entity);

	/**
	 * update the entity at the specified row
	 *
	 * @param entity
	 * @param row
	 */
	public void updateRow(AbstractEntity entity, int row) {
		List<Object> rowVector = getRow(entity);
		int col = 0;
		for (Object val : rowVector) {
			tableModel.setValueAt(val, row, col);
			++col;
		}
		tableModel.fireTableDataChanged();
	}

	/**
	 * get the row content
	 *
	 * @param entity
	 * @return
	 */
	public List<Object> getRow(AbstractEntity entity) {
		//LOG.trace(TT + ".getRow(" + AbstractEntity.trace(entity) + ") table=" + getName());
		List<Object> cols = new ArrayList<>();
		if (entity == null) {
			return (cols);
		}
		cols.add(entity.getId());
		if (!(entity instanceof Endnote)) {
			cols.add(entity.getName());
		}
		return cols;
	}

	/**
	 * get the common informations
	 *
	 * @param row
	 * @param entity
	 */
	public static void getRowEnd(List<Object> row, AbstractEntity entity) {
		String c = entity.getCommon();
		if (c.length() < 3) {
			c = c + "000";
		}
		if (c.charAt(0) == '1') {
			row.add(TextUtil.ellipsize(Html.htmlToText(entity.getDescription()), 64));
		}
		if (c.charAt(1) == '1') {
			row.add(TextUtil.ellipsize(Html.htmlToText(entity.getNotes()), 64));
		}
		if (c.charAt(2) == '1') {
			row.add(TextUtil.ellipsize(Assistant.toText(entity.getAssistant()), 64));
		}
	}

	/**
	 * update the entity from an event
	 *
	 * @param evt
	 */
	protected void updateEntity(PropertyChangeEvent evt) {
		//LOG.trace(TT+".updateEntity(evt="+evt.toString()+")");
		AbstractEntity entity = (AbstractEntity) evt.getNewValue();
		updateEntity(entity);
	}

	/**
	 * update an entity
	 *
	 * @param entity
	 */
	protected void updateEntity(AbstractEntity entity) {
		//LOG.trace(TT+".updateEntity(entity="+entity.toString()+")");
		fillTable();
		ListSelectionModel md = table.getSelectionModel();
		md.setSelectionInterval(currentRow, currentRow);
	}

	/**
	 * sort this table in order up
	 *
	 * @param evt
	 */
	protected void orderUpEntity(PropertyChangeEvent evt) {
	}

	/**
	 * sort this table in order down
	 *
	 * @param evt
	 */
	protected void orderDownEntity(PropertyChangeEvent evt) {
	}

	/**
	 * sort this table by the given column index
	 *
	 * @param col
	 */
	protected void sortByColumn(int col) {
		DefaultRowSorter<?, ?> sorter = ((DefaultRowSorter<?, ?>) table.getRowSorter());
		ArrayList<SortKey> list = new ArrayList<>();
		list.add(new RowSorter.SortKey(col, SortOrder.ASCENDING));
		sorter.setSortKeys(list);
		sorter.sort();
	}

	/**
	 * new entity action from an event
	 *
	 * @param evt
	 */
	protected void newEntity(PropertyChangeEvent evt) {
		AbstractEntity entity = (AbstractEntity) evt.getNewValue();
		newEntity(entity);
	}

	/**
	 * new entity action
	 *
	 * @param entity
	 */
	protected void newEntity(AbstractEntity entity) {
		//List<Object> cols = getRow(entity);
		fillTable();
		for (int row = 0; row < tableModel.getRowCount(); ++row) {
			AbstractEntity e = getEntityFromRow(row);
			if (e == null) {
				return;
			}
			if (e.getId().equals(entity.getId())) {
				table.setRowSelectionInterval(row, row);
				table.scrollRectToVisible(table.getCellRect(row, 0, true));
			}
		}
	}

	/**
	 * delete entity action from an event
	 *
	 * @param evt
	 */
	protected synchronized void deleteEntity(PropertyChangeEvent evt) {
		AbstractEntity entity = (AbstractEntity) evt.getOldValue();
		deleteEntity(entity);
	}

	/**
	 * delete entity action
	 *
	 * @param entity
	 */
	protected synchronized void deleteEntity(AbstractEntity entity) {
		//LOG.trace(TT + ".deleteEntity(entity=" + (entity == null ? "null" : entity.getName()) + ")");
		if (entity == null) {
			return;
		}
		boolean ok = false;
		for (int row = 0; row < tableModel.getRowCount(); ++row) {
			AbstractEntity e = getEntityFromRow(row);
			if (e != null && e.getId().equals(entity.getId())) {
				tableModel.removeRow(row);
				ok = true;
				break;
			}
		}
		if (!ok) {
			fillTable();
		}
		tableModel.fireTableDataChanged();
	}

	/**
	 * get the entity from given row
	 *
	 * @param row
	 * @return
	 */
	protected synchronized AbstractEntity getEntityFromRow(int row) {
		//LOG.trace(TT+".getEntityFromRow(row=" + row + ")");
		if (row == -1 || row > table.getRowCount() - 1) {
			return null;
		}
		try {
			if (table.getRowSorter() == null) {
				return null;
			}
			int modelIndex = table.getRowSorter().convertRowIndexToModel(row);
			@SuppressWarnings("unchecked")
			List<List<Object>> dataVector = tableModel.getDataVector();
			Long id = -1L;
			Object obj = dataVector.get(modelIndex).get(0);
			if (obj instanceof Long) {
				id = (Long) dataVector.get(modelIndex).get(0);
			} else if (obj instanceof String) {
				id = Long.parseLong((String) dataVector.get(modelIndex).get(0));
			}
			return getEntity(id);
		} catch (NumberFormatException ex) {
			LOG.err("AbstractTable.getEntityFromRow(" + row + ")", ex);
		}
		return null;
	}

	/**
	 * Performed action
	 *
	 * @param evt
	 */
	@Override
	public synchronized void actionPerformed(ActionEvent evt) {
		//LOG.trace(TT+".actionPerformed(" + evt.toString() + ")");
		Component comp = (Component) evt.getSource();
		if (comp instanceof JComboBox) {
			fillTable();
			return;
		}
		int row = table.getSelectedRow();
		AbstractEntity e;
		if (comp instanceof JToggleButton) {
			JToggleButton bt = (JToggleButton) evt.getSource();
			if (bt.getName().equals("table.editable")) {
				table.setEditable(bt.isSelected());
			}
		} else if (comp instanceof JButton) {
			switch (comp.getName()) {
				case BT_EDIT:
					int r = table.getSelectedRow();
					if (r != -1) {
						sendSetEntityToEdit(r);
					}
					break;

				case BT_NEW:
					table.clearSelection();
					sendSetNewEntityToEdit(BookUtil.getNewEntity(mainFrame, type));
					break;

				case BT_COPY:
					e = (AbstractEntity) getEntityFromRow(row);
					if (e != null) {
						EntityUtil.copyEntity(mainFrame, e);
						mainFrame.setUpdated();
					}
					break;
				case BT_DELETE:
					if (table.getSelectedRowCount() == 1) {
						// delete one entity
						e = (AbstractEntity) getEntityFromRow(row);
						DeleteEntityAction del
								= new DeleteEntityAction(mainFrame, e);
						del.actionPerformed(null);
						break;
					}
					if (table.getSelectedRowCount() > 1) {
						// delete multiple entities
						List<AbstractEntity> entities = new ArrayList<>();
						int[] rows = table.getSelectedRows();
						for (int row2 : rows) {
							AbstractEntity entity = getEntityFromRow(row2);
							if (entity != null) {
								List<Long> readOnlyIds = EntityUtil.getReadOnlyIds(entity);
								if (!readOnlyIds.contains(entity.getId())) {
									entities.add(entity);
								}
							}
						}
						ConfirmDeleteDlg dlg = new ConfirmDeleteDlg(mainFrame, entities);
						showModalDialog(dlg, mainFrame, true);
						if (dlg.isCanceled()) {
							break;
						}
						sendDeleteEntities(entities);
						break;
					}
					break;
				default:
					break;
			}
		}
	}

	/**
	 * Value changed action
	 *
	 * @param evt
	 */
	@Override
	public void valueChanged(ListSelectionEvent evt) {
		//LOG.trace(TT + ".valueChanged(e=" + evt.toString() + ")");
		DefaultListSelectionModel selectionModel = (DefaultListSelectionModel) evt.getSource();
		int count = selectionModel.getMaxSelectionIndex() - selectionModel.getMinSelectionIndex() + 1;
		if (count > 1) {
			btEdit.setEnabled(false);
			btCopy.setEnabled(false);
			btDelete.setEnabled(allowMultiDelete);
			return;
		}
		int row = selectionModel.getMinSelectionIndex();
		currentEntity = getEntityFromRow(row);
		boolean b = true;
		if (currentEntity == null) {
			b = false;
		}
		if (btEdit != null) {
			btEdit.setEnabled(b);
		}
		if (btCopy != null) {
			btCopy.setEnabled(b);
		}
		if (btDelete != null) {
			btDelete.setEnabled(b);
		}
		mainFrame.getBookController().showInfo(currentEntity);
		setEnableExternal(currentEntity);
	}

	/**
	 * enable the button to call external editor
	 *
	 * @param entity
	 */
	private void setEnableExternal(AbstractEntity entity) {
		if (entity instanceof Scene && btExternal != null) {
			Path path = Paths.get(((Scene) entity).getOdf());
			btExternal.setEnabled(Files.exists(path) && Files.isRegularFile(path));
		}
	}

	/**
	 * external editor action
	 *
	 * @param row
	 */
	public void sendSetExternal(int row) {
		LOG.trace(TT + ".sendSetLibreOffice(" + row + ")");
		Scene scene = (Scene) getEntityFromRow(row);
		if (scene == null) {
			return;
		}
		String name = scene.getOdf();
		if (name == null || name.isEmpty()) {
			name = XEditorFile.getDefaultFilePath(mainFrame, scene.getName());
		}
		File file = new File(name);
		if (!file.exists()) {
			file = XEditorFile.createFile(mainFrame, scene);
			if (file == null) {
				return;
			}
			scene.setOdf(file.getAbsolutePath());
			ctrl.updateEntity(scene);
		}
		XEditorFile.launchExternal(file.getAbsolutePath());
	}

	/**
	 * set the current row
	 *
	 * @param entity
	 */
	public void setCurrentRow(AbstractEntity entity) {
		for (int i = 0; i < tableModel.getRowCount(); i++) {
			String id = (String) tableModel.getValueAt(i, 0);
			if (id.equals(entity.getId().toString())) {
				currentRow = i;
				table.getSelectionModel().setSelectionInterval(currentRow, currentRow);
			}
		}
	}

	@Override
	public void refresh() {
		saveTableDesign();
		table.getSelectionModel().removeListSelectionListener(this);
		super.refresh();
		table.getSelectionModel().removeListSelectionListener(this);
		loadTableDesign();
		table.setRowSelectionInterval(currentRow, currentRow);
		table.getSelectionModel().addListSelectionListener(this);
	}

	/**
	 * refresh all data
	 *
	 */
	public void refreshAll() {
		saveTableDesign();
		table.getSelectionModel().removeListSelectionListener(this);
		this.removeAll();
		initUi();
		loadTableDesign();
		table.getSelectionModel().addListSelectionListener(this);
	}

	/**
	 * Table change event (simple change selected row)
	 *
	 * @param e
	 */
	@Override
	public void tableChanged(TableModelEvent e) {
		//LOG.trace(TT + ".tableChanged(e=" + e.toString() + ")" + " for table=" + table.getName());
		int row = e.getFirstRow();
		if (row < 0 || row != e.getLastRow()) {
			return;
		}
		AbstractEntity c = (AbstractEntity) this.getEntityFromRow(row);
		if (c != null) {
			mainFrame.getBookController().showInfo(c);
		}
	}

	public void addCbCategories(List<String> categories,
			AbstractEntity selected, boolean empty, boolean all) {
		toolbar.add(new JLabel(I18N.getColonMsg("categories")));
		cbCategories = Ui.initComboBox("cbCategories", "", categories, selected, empty, all, this);
		toolbar.add(cbCategories);
	}

	/**
	 * panel to show the objective
	 *
	 */
	private static class ObjectifPanel extends JPanel {

		private final AbstractTable table;
		private JLabel lbTotal;
		private JLabel lbRealized;
		private Integer total = 0;

		public ObjectifPanel(AbstractTable table) {
			this.table = table;
			initUi();
		}

		private void initUi() {
			setLayout(new MigLayout(MIG.INS0));
			lbTotal = new JLabel("0");
			lbTotal.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			lbRealized = new JLabel("0%");
			lbRealized.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			add(lbTotal);
			add(lbRealized);
		}

		public void setTotal(Integer value) {
			lbTotal.setText(String.format("%,d", value));
			total = value;
		}

		public void setRealized(Integer value) {
			String str = "0%";
			if (total > 0) {
				float c = (value * 100), t = total;
				str = (String.format("%.2f%%", c / t));
				if (c / t >= 100) {
					lbRealized.setForeground(Color.GREEN);
				} else {
					lbRealized.setForeground(lbTotal.getForeground());
				}
			}
			lbRealized.setText(str);
		}
	}

	/**
	 * Mouse adapter class
	 *
	 */
	private class MyMouseAdapter extends MouseAdapter {

		@Override
		public void mouseClicked(MouseEvent e) {
			//LOG.trace(TT + ".MyMouseAdapter.mouseClicked(e...)");
			DefaultListSelectionModel selectionModel = (DefaultListSelectionModel) table.getSelectionModel();
			int row = selectionModel.getMinSelectionIndex();
			int count = selectionModel.getMaxSelectionIndex() - selectionModel.getMinSelectionIndex() + 1;
			if (e.getClickCount() == 2) {
				if (count > 1) {
					return;
				}
				if (table.getName().equals("ChapterTable")) {
					int column = ((JSTable) e.getSource()).columnAtPoint(e.getPoint());
				}
				sendSetEntityToEdit(row);
			} else if (e.getClickCount() == 1 && count == 1) {
				AbstractEntity ent = getEntityFromRow(row);
				if (ent != null) {
					ctrl.showInfo(ent);
					if (type == Book.TYPE.SCENE && btExternal != null) {
						setEnableExternal(ent);
					}
				}
			}

		}

		@Override
		public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
				if (type == Book.TYPE.INTERNAL) {
					return;
				}
				DefaultListSelectionModel selectionModel = (DefaultListSelectionModel) table.getSelectionModel();
				int count = selectionModel.getMaxSelectionIndex() - selectionModel.getMinSelectionIndex() + 1;
				if (count == 1) {
					showPopup(e);
				} else {
					showPopupMulti(e);
				}
			}
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			// empty
		}

		/**
		 * show the Popup menu
		 *
		 * @param e
		 */
		private void showPopup(MouseEvent e) {
			if (!(e.getSource() instanceof JSTable)) {
				return;
			}
			JSTable source = (JSTable) e.getSource();
			int row = source.rowAtPoint(e.getPoint());
			int column = source.columnAtPoint(e.getPoint());
			if (!source.isRowSelected(row)) {
				source.changeSelection(row, column, false, false);
			}
			AbstractEntity entity = getEntityFromRow(row);
			if (entity != null) {
				JPopupMenu popup = EntityUtil.createPopupMenu(mainFrame, entity, EntityUtil.WITH_CHRONO);
				if (entity instanceof Scene || entity instanceof Chapter) {
					popup.add(new JPopupMenu.Separator());
					JMenuItem it = new JMenuItem(I18N.getMsg("copy.to_phpbb"));
					it.setIcon(IconUtil.getIconSmall(ICONS.K.EDIT_COPY));
					it.addActionListener(evt -> copyTo(row));
					popup.add(it);
				}
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}

		/**
		 * show Popup menu for multiline selection
		 *
		 * @param e
		 */
		private void showPopupMulti(MouseEvent e) {
			//LOG.trace(TT + ".showPopupMulti() for table " + table.getName());
			if (table.getName().equals("ChapterTable")
					|| table.getName().equals("SceneTable")) {
				int min = table.getSelectedRow();
				int max = table.getSelectionModel().getMaxSelectionIndex();
				List<AbstractEntity> rows = new ArrayList<>();
				for (int i = min; i <= max; i++) {
					AbstractEntity ent = getEntityFromRow(i);
					if (table.isRowSelected(i)) {
						rows.add(ent);
					}
				}
				JPopupMenu popup = new JPopupMenu();
				JMenuItem item;
				if (table.getName().equals("SceneTable")) {
					item = new JMenuItem(I18N.getMsg("scenes.change"));
					item.addActionListener(evt -> sceneChangeStatus(rows));
					popup.add(item);
				}
				item = new JMenuItem(I18N.getMsg("delete"));
				item.setIcon(IconUtil.getIconSmall(ICONS.K.DELETE));
				//item.setEnabled(false);
				item.addActionListener(evt -> sendDeleteEntities(rows));
				popup.add(item);
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	/**
	 * change status and/or chapter for multiple scenes
	 */
	private void sceneChangeStatus(List<AbstractEntity> scenes) {
		if (scenes == null || scenes.isEmpty()) {
			return;
		}
		if (!ScenesChange.show(mainFrame, scenes) && mainFrame.getView(VIEWNAME.TREE).isLoaded()) {
			mainFrame.getBookController().refresh(mainFrame.getView(VIEWNAME.TREE));
		}
	}

	/**
	 * save table columns size and position
	 *
	 */
	public void saveTableDesign() {
		//LOG.trace("SbViewFactory.saveTableDesign(mainFrame, table="+table.toString());
		String tableKey = TABLE_HEADER + "_" + getName();
		try {
			String cols = getTableDesign();
			App.preferences.setString(tableKey, cols);
		} catch (Exception e) {
			LOG.err("SbViewFactory.saveTableDesign(mainFrame, table=" + getName() + ")", e);
		}
	}

	/**
	 * get a TABCOL for a table
	 *
	 * @return
	 */
	private String getTableDesign() {
		//LOG.trace(TT + ".getTableDesign() for " + getName());
		List<String> list = new ArrayList<>();
		int n = 0;
		for (int i = 0; i < table.getColumnCount(); i++) {
			TableColumn col = table.getColumnModel().getColumn(i);
			if (col.getPreferredWidth() > 0) {
				String l1 = (String) col.getHeaderValue();
				int modelIdx = col.getModelIndex();
				if (modelIdx < 0) {
					continue;
				}
				int idx = table.convertColumnIndexToView(modelIdx);
				if (idx < 0) {
					continue;
				}
				list.add(String.format("%s,%d,%d", l1, col.getPreferredWidth(), n++));
			}
		}
		return ListUtil.join(list, "#");
	}

	/**
	 * load table columns size and position
	 *
	 */
	public void loadTableDesign() {
		//LOG.trace(TT + ".loadTableDesign() for " + getName());
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		String tableKey = TABLE_HEADER + "_" + getName();
		String z = App.preferences.getString(tableKey, "");
		if (z == null || z.isEmpty() || z.startsWith("(")) {
			return;
		}
		String[] xlist = z.split("#");
		for (String scol : xlist) {
			if (!scol.isEmpty()) {
				String[] x = scol.split(",");
				for (AbsColumn sbc : columns) {
					if (sbc.toString().equals(x[0])) {
						sbc.setSize(Integer.parseInt(x[1]));
						sbc.setOrder(Integer.parseInt(x[2]));
					}
				}
			}
		}
		// restore size and order
		int nbcols = table.getColumnCount();
		for (int i = 0; i < nbcols; i++) {
			table.setColumnVisible(tableModel.getColumnName(i), false);
		}
		for (AbsColumn c : columns) {
			TableColumn col = table.getColumn(c.toString());
			if (col != null && c.getSize() > 0) {
				table.setColumnVisible(c.toString(), true);
				col.setMinWidth(5);
				col.setPreferredWidth(c.getSize());
			}
		}
		List<AbsColumn> sbc = new ArrayList<>(columns);
		Collections.copy(sbc, columns);
		Collections.sort(sbc, (AbsColumn e1, AbsColumn e2) -> e1.getOrder().compareTo(e2.getOrder()));
		for (AbsColumn c : sbc) {
			if (c.getSize() > 0) {
				//LOG.trace(c.toString() + " : i=" + c.getOrder());
				for (int i = 0; i < nbcols; i++) {
					String colname = (String) table.getColumnModel().getColumn(i).getHeaderValue();
					if (colname.equals(c.toString())
							&& c.getOrder() != i
							&& c.getOrder() < nbcols) {
						table.moveColumn(i, c.getOrder());
						break;
					}
				}
			}
		}
		if (!App.isDev()) {
			table.setColumnVisible("ID", false);
		}
	}

	/**
	 * dump content of the table columns (for dev only)
	 */
	public void dumpDesign() {
		String tableKey = TABLE_HEADER + "_" + getName();
		String z = App.preferences.getString(tableKey, "");
		LOG.trace("table " + table.getName() + "=\"" + z + "\"");

	}

}
